import numpy

from scipy import interpolate
from scipy import spatial
from scipy.interpolate import interp2d, RectBivariateSpline

import matplotlib.pylab as plt
import matplotlib as mpl

from srxraylib.plot.gol import plot_image, set_qt, plot

from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt

from oasys.util.oasys_util import write_surface_file

from oasys.widgets import congruence
import oasys.util.oasys_util as OU


def write_generic_h5_surface(s, xx, yy, filename='presurface.hdf5',subgroup_name="surface_file"):
    # import h5py
    # file = h5py.File(filename, 'w')
    # file[subgroup_name + "/X"] = xx
    # file[subgroup_name + "/Y"] = yy
    # file[subgroup_name + "/Z"] = s.T
    # file.close()
    # print("write_h5_surface: File for OASYS " + filename + " written to disk.")
    write_surface_file(s.T, xx, yy, filename, overwrite=True)
    print("write_h5_surface: File for OASYS " + filename + " written to disk.")


class FEA_File():
    def __init__(self,filename=""):
        self.filename = filename
        self.reset()

    def reset(self):
        self.Xundeformed = None  # 1D array
        self.Yundeformed = None  # 1D array
        self.Zundeformed = None  # 1D array

        self.Xdeformation = None  # 1D array
        self.Ydeformation = None  # 1D array
        self.Zdeformation = None  # 1D array

        self.triPi = None
        self.tri = None

        self.x_interpolated = None  # 1D array
        self.y_interpolated = None  # 1D array
        self.Z_INTERPOLATED = None  # 2D array

        self.file_in_type = None

    @classmethod
    def process_file(cls, filename_in, n_axis_0=301, n_axis_1=51,
                     filename_out="", invert_axes_names=False,
                     detrend=0, # 0=none 1(2)=straight line axis 0 (1), 3(4) best circle axis 0(1)
                     reset_height_method=0,
                     replicate_raw_data_flag=0, # 0=None, 1=axis0, 2=axis1, 3=both axis
                     file_in_type=0,skiprows=0,
                     do_plot=False):

        o1 = FEA_File(filename=filename_in)
        o1.load_multicolumn_file(skiprows=skiprows,file_in_type=file_in_type)


        o1.replicate_raw_data(replicate_raw_data_flag)



        o1.triangulate()

        if do_plot:
            o1.plot_triangulation()

        o1.interpolate(n_axis_0, n_axis_1)
        if do_plot:
            o1.plot_interpolated()

        if o1.does_interpolated_have_nan():
            o1.remove_borders_in_interpolated_data()

        if do_plot:
            o1.plot_surface_image()

        if detrend == 0:
            pass
        elif detrend == 1:
            o1.detrend_straight_line(axis=0)
        elif detrend == 2:
            o1.detrend_straight_line(axis=1)
        elif detrend == 3:
            o1.detrend_best_circle(axis=0)
        elif detrend == 5:
            o1.detrend_best_circle(axis=1)

        # o1.reset_height_to_minimum()

        if reset_height_method == 0:
            pass
        elif reset_height_method == 1:
            o1.reset_height_to_minimum()
        elif reset_height_method == 2:
            o1.reset_height_to_central_value()

        if do_plot:
            o1.plot_surface_image()

        if filename_out != "":
            o1.write_h5_surface(filename=filename_out, invert_axes_names=invert_axes_names)

        return o1

    def set_filename(self,filename):
        self.filename = filename

    def set_file_in_type(self, value):
        self.file_in_type = value

    def load_multicolumn_file(self,skiprows=0,factorX=1.0,factorY=1.0,factorZ=1.0,file_in_type=0):
        self.set_file_in_type(file_in_type)
        print("Reading file/url: %s" % self.filename)
        if file_in_type == 0: # ALS
            node = numpy.loadtxt(self.filename, skiprows=skiprows, dtype=numpy.float64)

            # Coordinates
            self.Xundeformed = factorX * node[:, 1]  # X=x in m
            self.Yundeformed = factorY * node[:, 2]  # Y=z in m
            self.Zundeformed = factorZ * node[:, 3]  # Z=uy vertical displacement in m

            self.Xdeformation = factorX * node[:, 4]  # X=x in m
            self.Ydeformation = factorY * node[:, 5]  # Y=z in m
            self.Zdeformation = factorZ * node[:, 6]  # Z=uy vertical displacement in m
        elif file_in_type == 1: # ESRF
            node = numpy.loadtxt(self.filename, skiprows=skiprows, dtype=numpy.float64, delimiter=',')

            # Coordinates
            self.Xundeformed = factorX * node[:, 0]  # X=x in m
            self.Yundeformed = factorY * node[:, 1]  # Y=z in m
            self.Zundeformed = factorZ * node[:, 2]  # Z=uy vertical displacement in m

            self.Xdeformation = factorX * node[:, 3]  # X=x in m
            self.Ydeformation = factorY * node[:, 4]  # Y=z in m
            self.Zdeformation = factorZ * node[:, 5]  # Z=uy vertical displacement in m

        elif file_in_type == 2: # HDF5 Oasys surface file

            if self.filename[0:4] == "http" or self.filename[0:3] == "ftp":
                import urllib.request
                filehandle, _ = urllib.request.urlretrieve(self.filename)
                print("Reading local file doenloaded from url: %s" % filehandle)
            else:
                filehandle = self.filename

            self.surface_file_name = congruence.checkDir(filehandle)
            # if not os.path.isfile(file_name): raise ValueError("File " + file_name + " not existing")
            # import h5py
            # file = h5py.File(filehandle, 'r')
            # xx = file["surface_file" + "/X"][()]
            # yy = file["surface_file" + "/Y"][()]
            # zz = file["surface_file" + "/Z"][()]
            # file.close()
            # return xx.copy(), yy.copy(), zz.copy()

            xx, yy, zz = OU.read_surface_file(filehandle)


            ZZ = zz.copy().T
            XX = numpy.outer(xx, numpy.ones_like(yy))
            YY = numpy.outer(numpy.ones_like(xx), yy)

            # Coordinates


            self.Xdeformation = factorX * XX.flatten()  # X=x in m
            self.Ydeformation = factorY * YY.flatten()  # Y=z in m
            self.Zdeformation = factorZ * ZZ.flatten()  # Z=uy vertical displacement in m

            self.Xundeformed = numpy.zeros_like(self.Xdeformation) # factorX * XX.flatten() * 0.0 # X=x in m
            self.Yundeformed = numpy.zeros_like(self.Ydeformation) # factorY * YY.flatten() * 0.0 # Y=z in m
            self.Zundeformed = numpy.zeros_like(self.Zdeformation) # factorZ * ZZ.flatten() * 0.0  # Z=uy vertical displacement in m

            #
            # this file type does not need interpolation as it comes in a regular grid
            #

            self.x_interpolated = factorX * xx
            self.y_interpolated = factorY * yy
            self.Z_INTERPOLATED = factorZ * ZZ

    def Xdeformed(self):
        return self.Xundeformed + self.Xdeformation

    def Ydeformed(self):
        return self.Yundeformed + self.Ydeformation

    def Zdeformed(self):
        return self.Zundeformed + self.Zdeformation


    def get_deformed(self):
        return self.Xdeformed(),self.Ydeformed(),self.Zdeformed()

    def get_undeformed(self):
        return self.Xundeformed,self.Yundeformed,self.Zundeformed

    def get_deformation(self):
        return self.Xdeformation,self.Ydeformation,self.Zdeformation

    def get_limits_undeformed(self):

        print("X undeformed limits: ", self.Xundeformed.min(), self.Xundeformed.max())
        print("Y undeformed limits: ", self.Yundeformed.min(), self.Yundeformed.max())
        print("Z undeformed limits: ", self.Zundeformed.min(), self.Zundeformed.max())

        return self.Xundeformed.min(), self.Xundeformed.max(), \
               self.Yundeformed.min(), self.Yundeformed.max(), \
               self.Zundeformed.min(), self.Zundeformed.max()

    def get_limits_deformation(self):

        print("Xdeformation limits: ", self.Xdeformation.min(), self.Xdeformation.max())
        print("Ydeformation limits: ", self.Ydeformation.min(), self.Ydeformation.max())
        print("Zdeformation limits: ", self.Zdeformation.min(), self.Zdeformation.max())

        return self.Xdeformation.min(), self.Xdeformation.max(), \
               self.Ydeformation.min(), self.Ydeformation.max(), \
               self.Zdeformation.min(), self.Zdeformation.max()

    def get_limits_deformed(self):

        print("X deformed limits: ", self.Xdeformed().min(), self.Xdeformed().max())
        print("Y deformed limits: ", self.Ydeformed().min(), self.Ydeformed().max())
        print("Z deformed limits: ", self.Zdeformed().min(), self.Zdeformed().max())

        return self.Xdeformed().min(), self.Xdeformed().max(), \
               self.Ydeformed().min(), self.Ydeformed().max(), \
               self.Zdeformed().min(), self.Zdeformed().max()

    def get_dimensions(self):
        return self.Xundeformed.size, self.Yundeformed.size, self.Zundeformed.size

    def replicate_raw_data(self,flag):

        if flag == 0: # nothing
            return
        elif flag == 1: # axis 0
            self.Xundeformed = numpy.concatenate((-self.Xundeformed, self.Xundeformed))
            self.Yundeformed = numpy.concatenate((self.Yundeformed, self.Yundeformed))
            self.Zundeformed = numpy.concatenate((self.Zundeformed, self.Zundeformed))
            self.Xdeformation = numpy.concatenate((-self.Xdeformation, self.Xdeformation))
            self.Ydeformation = numpy.concatenate((self.Ydeformation, self.Ydeformation))
            self.Zdeformation = numpy.concatenate((self.Zdeformation, self.Zdeformation))
        elif flag == 2: # axis 1
            self.Xundeformed = numpy.concatenate((self.Xundeformed, self.Xundeformed))
            self.Yundeformed = numpy.concatenate((-self.Yundeformed, self.Yundeformed))
            self.Zundeformed = numpy.concatenate((self.Zundeformed, self.Zundeformed))
            self.Xdeformation = numpy.concatenate((self.Xdeformation, self.Xdeformation))
            self.Ydeformation = numpy.concatenate((-self.Ydeformation, self.Ydeformation))
            self.Zdeformation = numpy.concatenate((self.Zdeformation, self.Zdeformation))
        elif flag == 3: # both axes
            self.Xundeformed = numpy.concatenate((-self.Xundeformed, self.Xundeformed))
            self.Yundeformed = numpy.concatenate((self.Yundeformed, self.Yundeformed))
            self.Zundeformed = numpy.concatenate((self.Zundeformed, self.Zundeformed))
            self.Xdeformation = numpy.concatenate((-self.Xdeformation, self.Xdeformation))
            self.Ydeformation = numpy.concatenate((self.Ydeformation, self.Ydeformation))
            self.Zdeformation = numpy.concatenate((self.Zdeformation, self.Zdeformation))

            self.Xundeformed = numpy.concatenate((self.Xundeformed, self.Xundeformed))
            self.Yundeformed = numpy.concatenate((-self.Yundeformed, self.Yundeformed))
            self.Zundeformed = numpy.concatenate((self.Zundeformed, self.Zundeformed))
            self.Xdeformation = numpy.concatenate((self.Xdeformation, self.Xdeformation))
            self.Ydeformation = numpy.concatenate((-self.Ydeformation, self.Ydeformation))
            self.Zdeformation = numpy.concatenate((self.Zdeformation, self.Zdeformation))


    def triangulate(self):
        # triangulation
        self.triPi = numpy.array([self.Xdeformed(), self.Ydeformed()]).transpose()
        self.tri = spatial.Delaunay(self.triPi)

    def plot_triangulation(self,show=True):
        fig = plt.figure()
        plt.triplot(self.Xdeformed(), self.Ydeformed() , self.tri.simplices.copy())
        plt.plot(self.Xdeformed(), self.Ydeformed(), "or", label = "Data")
        plt.grid()
        plt.legend()
        plt.title("triangulation")
        plt.xlabel("x")
        plt.ylabel("y")
        if show:
            plt.show()
        return fig

    def get_Xinterpolated_mesh(self):
        return numpy.outer(self.x_interpolated,numpy.ones_like(self.y_interpolated))

    def get_Yinterpolated_mesh(self):
        return numpy.outer(numpy.ones_like(self.x_interpolated),self.y_interpolated)


    def interpolate(self,nx,ny,remove_nan=0):
        """

        :param nx:
        :param ny:
        :param remove_nan: 0=No, 1=Yes (replace with minimum height) 2=Yes (replace with 0)
        :return:
        """

        # if input file is OASYS h5, the grid is regular so no need of triangulation

        if self.file_in_type == 2:
            if nx <= 0 or ny <=0:
                # nothing to do
                print("Skip interpolation")
            else: # interpolate one by one along Y
                lim = self.get_limits_deformed()
                print("interpolation limits: xmin, xmax, ymin, ymax", lim[0:4])
                x_interpolated = numpy.linspace(lim[0], lim[1], nx)
                y_interpolated = numpy.linspace(lim[2], lim[3], ny)

                f = RectBivariateSpline(self.x_interpolated,
                             self.y_interpolated,
                             self.Z_INTERPOLATED)
                Z_INTERPOLATED = f(x_interpolated, y_interpolated)

                print("original dimensions f0r interpolation", self.Z_INTERPOLATED.shape)

                print("interpolated dimensions", Z_INTERPOLATED.shape)

                # overwrite interpolated data
                self.x_interpolated = x_interpolated
                self.y_interpolated = y_interpolated
                self.Z_INTERPOLATED = Z_INTERPOLATED



        else:


            if self.tri is None:
                self.triangulate()

            lim = self.get_limits_deformed()
            self.x_interpolated = numpy.linspace(lim[0],lim[1],nx)
            self.y_interpolated = numpy.linspace(lim[2],lim[3],ny)

            X_INTERPOLATED =  self.get_Xinterpolated_mesh()
            Y_INTERPOLATED =  self.get_Yinterpolated_mesh()

            self.P = numpy.array([X_INTERPOLATED.flatten(), Y_INTERPOLATED.flatten() ]).transpose()

            if remove_nan ==2:
                self.Z_INTERPOLATED = interpolate.griddata(self.triPi, self.Zdeformed(), self.P, rescale=True, method = "cubic", fill_value=0.0 ).reshape([nx,ny])
            elif remove_nan ==1:
                self.Z_INTERPOLATED = interpolate.griddata(self.triPi, self.Zdeformed(), self.P, rescale=True, method = "cubic", fill_value=self.Zdeformed().min() ).reshape([nx,ny])
            elif remove_nan == 0:
                self.Z_INTERPOLATED = interpolate.griddata(self.triPi, self.Zdeformed(), self.P, rescale=True, method="cubic").reshape([nx, ny])


    def plot_interpolated(self, show=True):
        fig = plt.figure()
        plt.contourf(self.get_Xinterpolated_mesh(), self.get_Yinterpolated_mesh(), self.Z_INTERPOLATED, 50, cmap = mpl.cm.jet)
        plt.colorbar()
        plt.contour(self.get_Xinterpolated_mesh(), self.get_Yinterpolated_mesh(), self.Z_INTERPOLATED, 20, colors = "k")
        plt.plot(self.Xdeformed(), self.Ydeformed(), "or", label = "Data")
        plt.legend()
        # plt.title = "Interpolated"  <---- THIS MAKES ERROR IN THE NEXT PLOT!!!!!!!!!!!!!!!!!
        plt.grid()
        if show:
            plt.show()
        return fig



    def plot_surface_image(self,invert_axes_names=False):
        if invert_axes_names:
            plot_image(self.Z_INTERPOLATED,self.x_interpolated,self.y_interpolated ,title="file: %s, axes names INVERTED from ANSYS"%self.filename,
                       xtitle="Y (%d pixels, max:%f)"%(self.x_interpolated.size,self.x_interpolated.max()),
                       ytitle="X (%d pixels, max:%f)"%(self.y_interpolated.size,self.y_interpolated.max()) )
        else:
            plot_image(self.Z_INTERPOLATED,self.x_interpolated,self.y_interpolated,title="file: %s, axes as in ANSYS"%self.filename,
                       xtitle="X (%d pixels, max:%f)"%(self.x_interpolated.size,self.x_interpolated.max()),
                       ytitle="Y (%d pixels, max:%f)"%(self.y_interpolated.size,self.y_interpolated.max()) )


    def does_interpolated_have_nan(self):
        return numpy.isnan(self.Z_INTERPOLATED).sum() > 0

    def remove_borders_in_interpolated_data(self):
        self.x_interpolated = self.x_interpolated[1:-2].copy()
        self.y_interpolated = self.y_interpolated[1:-2].copy()
        self.Z_INTERPOLATED = self.Z_INTERPOLATED[1:-2,1:-2].copy()

    def detrend_straight_line(self,axis=0,fitting_domain_ratio=0.5):
        if axis == 0:
            xm = self.x_interpolated.copy()
            zm = self.Z_INTERPOLATED[:,self.y_interpolated.size//2]
        elif axis == 1:
            xm = self.y_interpolated.copy()
            zm = self.Z_INTERPOLATED[self.x_interpolated.size // 2, :]

        zm.shape = -1

        icut = numpy.argwhere( numpy.abs(xm) < (numpy.max((-xm[0],xm[-1])) * fitting_domain_ratio))
        if len(icut) <=5:
            raise Exception("Not enough points for fitting.")

        xcut = xm[icut]
        zmcut = zm[icut]

        xcut.shape = -1
        zmcut.shape = -1

        # plot(xcut, zmcut, xm, zm, legend=["cut", "original"])

        print( numpy.argwhere(numpy.isnan(self.Z_INTERPOLATED)) )
        print("Fitting interval: [%g,%g]" % (xcut[0],xcut[-1]))

        coeff = numpy.polyfit(xcut.copy(), zmcut.copy(), deg=1)

        zfit = coeff[0] * xm  + coeff[1]

        print("Detrending straight line (axis=%d): zfit = %g * coordinate + %g " % (axis, coeff[1], coeff[0]))

        # plot(xcut, zmcut, xm, zfit, legend=["data","fit"])
        # plot(xcut, zmcut, xm, zfit, legend=["cut","fit"],yrange=[-0.000015,0.000005])

        if axis ==0:
            for i in range(self.Z_INTERPOLATED.shape[1]):
                self.Z_INTERPOLATED[:,i] -= zfit
        elif axis == 1:
            for i in range(self.Z_INTERPOLATED.shape[0]):
                self.Z_INTERPOLATED[i,:] -= zfit

    def detrend_best_circle(self,axis=0,fitting_domain_ratio=0.5):
        if axis == 0:
            xm = self.x_interpolated.copy()
            zm = self.Z_INTERPOLATED[:,self.y_interpolated.size//2]
        elif axis == 1:
            xm = self.y_interpolated.copy()
            zm = self.Z_INTERPOLATED[self.x_interpolated.size // 2, :]

        zm.shape = -1

        # icut = numpy.argwhere( numpy.abs(xm) < (numpy.max((-xm[0],xm[-1])) * fitting_domain_ratio))
        icut = numpy.argwhere(numpy.abs(xm) <= fitting_domain_ratio)
        if len(icut) <=5:
            raise Exception("Not enough points for fitting.")

        xcut = xm[icut]
        zmcut = zm[icut]

        xcut.shape = -1
        zmcut.shape = -1

        # plot(xcut, zmcut, xm, zm, legend=["cut", "original"])

        print( numpy.argwhere(numpy.isnan(self.Z_INTERPOLATED)) )
        print("Fitting interval: [%g,%g] (using %d points)" % (xcut[0],xcut[-1],xcut.size))

        coeff = numpy.polyfit(xcut, numpy.gradient(zmcut,xcut), deg=1)

        # zfit = coeff[0] * xm  + coeff[1]
        radius = 1 / coeff[0]
        print("Detrending straight line on sloped (axis=%d): zfit = %g * coordinate + %g " % (axis, coeff[1], coeff[0]))
        print("Radius of curvature: %g m" % (1.0 / coeff[0]))

        if radius >= 0:
            zfit = radius - numpy.sqrt(radius ** 2 - xm ** 2)
        else:
            zfit = radius + numpy.sqrt(radius ** 2 - xm ** 2)

        # plot(xm, zfit, legend=["fit"])
        # plot(xcut, zmcut, xm, zfit, legend=["cut","fit"],yrange=[-0.000015,0.000005])

        if axis ==0:
            for i in range(self.Z_INTERPOLATED.shape[1]):
                self.Z_INTERPOLATED[:,i] -= zfit
        elif axis == 1:
            for i in range(self.Z_INTERPOLATED.shape[0]):
                self.Z_INTERPOLATED[i,:] -= zfit

        return xm, zfit

    def reset_height_to_minimum(self):
        self.Z_INTERPOLATED -= self.Z_INTERPOLATED.min()

    def reset_height_to_central_value(self):
        self.Z_INTERPOLATED -= self.Z_INTERPOLATED[self.Z_INTERPOLATED.shape[0]//2,self.Z_INTERPOLATED.shape[1]//2]


    def write_h5_surface(self,filename='presurface.hdf5',invert_axes_names=False):
        if invert_axes_names:
            write_generic_h5_surface(self.Z_INTERPOLATED.T, self.y_interpolated, self.x_interpolated, filename=filename)
        else:
            write_generic_h5_surface(self.Z_INTERPOLATED, self.x_interpolated, self.y_interpolated, filename=filename)


    def gaussian_filter(self,sigma_axis0=10,sigma_axis1=10):
        from scipy.ndimage import gaussian_filter
        self.Z_INTERPOLATED = gaussian_filter(self.Z_INTERPOLATED, (sigma_axis0,sigma_axis1),
                        order=0, output=None, mode='nearest', cval=0.0, truncate=4.0)

def surface_plot(xs,ys,zs):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')


    # For each set of style and range settings, plot n random points in the box
    # defined by x in [23, 32], y in [0, 100], z in [zlow, zhigh].
    # for m, zlow, zhigh in [('o', -50, -25), ('^', -30, -5)]:
    for m, zlow, zhigh in [('o', zs.min(), zs.max())]:
        ax.scatter(xs, ys, zs, marker=m)

    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')

    plt.show()



if __name__ == "__main__":




    # set_qt()


    # o1 = FEA_File.process_file("/home/manuel/OASYS1.2/alsu-scripts/ANSYS/s4.txt", n_axis_0=301, n_axis_1=51,
    #              filename_out="/home/manuel/Oasys/s4.h5", invert_axes_names=True,
    #              detrend=1, reset_height_method=1, do_plot=False)

    # o1 = FEA_File()
    # o1.set_filename("C:/Users/Manuel/Oasys/dispCOSMIC_M1_H_XOPPY.txt")
    # o1.load_multicolumn_file()
    #
    # X,Y,Z = o1.get_deformed()
    # # X, Y, Z = o1.get_undeformed()
    # surface_plot( X,Y,Z)

    # o1.plot_triangulation()
    # o1.plot_interpolated()
    # o1.plot_surface_image()


    # o1 = FEA_File.process_file("C:/Users/Manuel/Oasys/TENDER DCM Performance/disp2000.txt", n_axis_0=801, n_axis_1=801,
    #              filename_out="", invert_axes_names=True,
    #              detrend=1, reset_height_method=2,
    #              replicate_raw_data_flag=3,do_plot=False)
    # o1.plot_surface_image()
    # # plot(o1.x_interpolated, o1.Z_INTERPOLATED[:,o1.y_interpolated.size//2]) #, o1.y_interpolated
    # x0, y0, z0 = o1.get_undeformed()
    # x, y, z = o1.get_deformed()
    # X = []
    # Z = []
    # numpy.set_printoptions(precision=17)
    #
    # for i in range(x.size):
    #     if numpy.abs(y[i]) ==0:
    #         X.append(x[i])
    #         Z.append(z[i])
    #         if numpy.abs(x[i]) < 50e-6:
    #             print("line %d x:%g z: %g %g " % (i, x[i], z0[i], z[i]))
    # X = numpy.array(X)
    # Z = numpy.array(Z)
    # plot(X, Z,
    #      o1.x_interpolated, o1.Z_INTERPOLATED[:,o1.y_interpolated.size//2],marker=['.','.'],
    #      linestyle=["",""],legend=["raw","interpolated"],xrange=[-250e-6,1550e-6])




    # o1 = FEA_File.process_file("73water_side_cooled_notches_best_LH.txt", n_axis_0=1001, n_axis_1=101,
    #              filename_out="/home/manuel/Oasys/water_side_cooled_notches_best_LH.h5", invert_axes_names=True,
    #              detrend=1, reset_height_method=2,
    #              replicate_raw_data_flag=3,do_plot=False)
    #
    # o1 = FEA_File.process_file("73water_side_cooled_notches_best_LV.txt", n_axis_0=1001, n_axis_1=101,
    #              filename_out="/home/manuel/Oasys/water_side_cooled_notches_best_LV.h5", invert_axes_names=True,
    #              detrend=0, reset_height_method=0,
    #              replicate_raw_data_flag=3,do_plot=False)

    fea_file_object = FEA_File()
    fea_file_object.set_filename("/users/srio/OASYS1.2/shadow3-scripts/METROLOGY/ring256.h5")
    fea_file_object.load_multicolumn_file(skiprows=0,
                                               file_in_type=2,
                                               factorX=1.0,
                                               factorY=1.0,
                                               factorZ=1.0)

    fea_file_object.interpolate(0, 0, remove_nan=0)
    print(fea_file_object.x_interpolated.shape,
          fea_file_object.y_interpolated.shape,
          fea_file_object.Z_INTERPOLATED.shape,)
    plot_image(fea_file_object.Z_INTERPOLATED, fea_file_object.x_interpolated, fea_file_object.y_interpolated,
               aspect='auto', title="No interpolation", show=0)


    fea_file_object.interpolate(70, 800, remove_nan=0)
    # fea_file_object.plot_surface_image()
    print(fea_file_object.x_interpolated.shape,
          fea_file_object.y_interpolated.shape,
          fea_file_object.Z_INTERPOLATED.shape,)

    plot_image(fea_file_object.Z_INTERPOLATED, fea_file_object.x_interpolated, fea_file_object.y_interpolated,
               aspect='auto', title="Interpolated")




import numpy
import sys

from PyQt5.QtWidgets import QMessageBox

from orangewidget import gui
from orangewidget.settings import Setting

from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.util.oasys_util import TriggerIn, EmittingStream



from syned.beamline.element_coordinates import ElementCoordinates
from syned.beamline.beamline_element import BeamlineElement
from syned.widget.widget_decorator import WidgetDecorator

from wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D

from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.esrf.wofry.widgets.gui.ow_optical_element_1d import OWWOOpticalElement1D # to be moved
from orangecontrib.esrf.wofry.util.mirror import WOMirror1D # to be moved to wofryimpl



class OWMirror1D(OWWOOpticalElement1D):

    name = "Mirror 1D"
    id = "WofryMirror1D"
    description = "Mirror 1D"
    icon = "icons/reflector_grazing1D.png"
    priority = 10

    category = "Wofry Wavefront Propagation"
    keywords = ["data", "file", "load", "read", "grazing"]

    outputs = [{"name":"WofryData",
                "type":WofryData,
                "doc":"WofryData",
                "id":"WofryData"},
               {"name":"Trigger",
                "type": TriggerIn,
                "doc":"Feedback signal to start a new beam simulation",
                "id":"Trigger"}]

    inputs = [("WofryData", WofryData, "set_input"),
              ("GenericWavefront1D", GenericWavefront1D, "set_input"),
              ("DABAM 1D Profile", numpy.ndarray, "receive_dabam_profile"),
              WidgetDecorator.syned_input_data()[0]]

    grazing_angle_in = Setting(1.5e-3)

    shape = Setting(1)
    flip = Setting(0)
    p_focus = Setting(1.0)
    q_focus = Setting(1.0)
    error_flag = Setting(0)
    error_file = Setting("<none>")
    error_file_oversampling_factor = Setting(1.0)
    mirror_length = Setting(0.2)
    mirror_points = Setting(500)
    write_profile = Setting(0)
    write_input_wavefront = Setting(0)


    input_data = None
    titles = ["Wavefront 1D Intensity", "Wavefront 1D Phase","Wavefront Real(Amplitude)","Wavefront Imag(Amplitude)","O.E. Profile"]


    def __init__(self):
        super().__init__(is_automatic=True, show_view_options=True, show_script_tab=True)

    def draw_specific_box(self):

        box_reflector = oasysgui.widgetBox(self.tab_bas, "Reflector", addSpace=False, orientation="vertical")

        oasysgui.lineEdit(box_reflector, self, "grazing_angle_in", "Grazing incidence angle [rad]",
                          labelWidth=300, valueType=float, orientation="horizontal")

        gui.comboBox(box_reflector, self, "shape", label="Reflector shape",
                     items=["Flat","Circle","Ellipse","Parabola"], sendSelectedValue=False, orientation="horizontal",
                     callback=self.set_visible)

        gui.comboBox(box_reflector, self, "flip", label="Flip mirror (up-down,left-right)",
                     items=["No","Yes"], sendSelectedValue=False, orientation="horizontal")

        self.box_focal_id = oasysgui.widgetBox(box_reflector, "", addSpace=True, orientation="vertical")
        oasysgui.lineEdit(self.box_focal_id, self, "p_focus", "Focal entrance arm [m]",
                          labelWidth=300, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.box_focal_id, self, "q_focus", "Focal exit arm [m]",
                          labelWidth=300, valueType=float, orientation="horizontal")

        gui.comboBox(box_reflector, self, "error_flag", label="Add profile deformation",
                     items=["No","Yes (from file)"],
                     callback=self.set_visible,
                     sendSelectedValue=False, orientation="horizontal")

        self.file_box_id = oasysgui.widgetBox(box_reflector, "", addSpace=True, orientation="vertical")
        file_box_id2 = oasysgui.widgetBox(self.file_box_id, "", addSpace=True, orientation="horizontal")
        self.error_file_id = oasysgui.lineEdit(file_box_id2, self, "error_file", "Error file X[m] Y[m]",
                                                    labelWidth=120, valueType=str, orientation="horizontal")
        gui.button(file_box_id2, self, "...", callback=self.set_error_file)

        oasysgui.lineEdit(self.file_box_id, self, "error_file_oversampling_factor", "Oversampling factor (>=1)",
                          labelWidth=300, valueType=float, orientation="horizontal")


        self.mirror_box_id = oasysgui.widgetBox(box_reflector, "", addSpace=True, orientation="vertical")
        oasysgui.lineEdit(self.mirror_box_id, self, "mirror_length", "Mirror length [m]",
                          labelWidth=300, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.mirror_box_id, self, "mirror_points", "Points on mirror",
                          labelWidth=300, valueType=int, orientation="horizontal")

        gui.comboBox(box_reflector, self, "write_profile", label="Dump profile to file",
                     items=["No","Yes [reflector_profile1D.dat]"], sendSelectedValue=False, orientation="horizontal")

        self.set_visible()



    def set_visible(self):
        self.file_box_id.setVisible(self.error_flag)
        self.box_focal_id.setVisible(self.shape)
        self.mirror_box_id.setVisible(self.error_flag == 0)

    def set_error_file(self):
        self.error_file_id.setText(oasysgui.selectFileFromDialog(self, self.error_file, "Open file with profile error"))

    def check_fields(self):
        self.grazing_angle_in = congruence.checkStrictlyPositiveNumber(self.grazing_angle_in, "Grazing incidence angle")
        self.p_focus = congruence.checkNumber(self.p_focus, "p focus")
        self.q_focus = congruence.checkNumber(self.q_focus, "q focus")
        self.error_file = congruence.checkFileName(self.error_file)
        self.error_file_oversampling_factor = congruence.checkStrictlyPositiveNumber(self.error_file_oversampling_factor)

    def receive_syned_data(self):
        raise Exception(NotImplementedError)

    def set_input(self, wofry_data):
        if not wofry_data is None:
            if isinstance(wofry_data, WofryData):
                self.input_data = wofry_data
            else:
                self.input_data = WofryData(wavefront=wofry_data)

            if self.is_automatic_execution:
                self.propagate_wavefront()

    def receive_dabam_profile(self, dabam_profile):
        if not dabam_profile is None:
            try:
                file_name = "dabam_profile_" + str(id(self)) + ".dat"

                file = open(file_name, "w")

                for element in dabam_profile:
                    file.write(str(element[0]) + " " + str(element[1]) + "\n")

                file.flush()
                file.close()

                self.error_flag = 1
                self.error_file = file_name
                self.set_visible()

            except Exception as exception:
                QMessageBox.critical(self, "Error", exception.args[0], QMessageBox.Ok)

                if self.IS_DEVELOP: raise exception

    def get_optical_element(self):
        if self.error_flag == 0:
            error_file = ""
            mirror_length = self.mirror_length
            mirror_points = self.mirror_points
        else:
            error_file = self.error_file
            mirror_length = 0
            mirror_points = 0

        return WOMirror1D.create_from_keywords(
                    name                         =self.oe_name,
                    shape=self.shape,
                    flip=self.flip,
                    p_focus=self.p_focus,
                    q_focus=self.q_focus,
                    grazing_angle_in=self.grazing_angle_in,
                    p_distance=self.p,
                    q_distance=self.q,
                    zoom_factor=self.magnification_x,
                    error_flag=self.error_flag,
                    error_file=error_file,
                    error_file_oversampling_factor=self.error_file_oversampling_factor,
                    mirror_length=mirror_length,
                    mirror_points=mirror_points,
                    write_profile=self.write_profile)


    def propagate_wavefront(self):
        self.progressBarInit()

        self.wofry_output.setText("")

        sys.stdout = EmittingStream(textWritten=self.writeStdOut)

        if self.input_data is None: raise Exception("No Input Data")

        self.check_data()

        # propagation to o.e.

        input_wavefront  = self.input_data.get_wavefront()
        beamline         = self.input_data.get_beamline().duplicate()

        optical_element = self.get_optical_element()
        optical_element.name = self.oe_name if not self.oe_name is None else self.windowTitle()

        beamline_element = BeamlineElement(optical_element=optical_element,
                                           coordinates=ElementCoordinates(p=0.0, # to avoid using standard propagators
                                                                          q=0.0, # to avoid using standard propagators
                                                                          angle_radial=numpy.radians(self.angle_radial),
                                                                          angle_azimuthal=numpy.radians(self.angle_azimuthal)))

        beamline.append_beamline_element(beamline_element)

        self.setStatusMessage("Begin Propagation")

        output_wavefront = optical_element.applyOpticalElement(input_wavefront.duplicate())
        self.setStatusMessage("Propagation Completed")

        self.wavefront_to_plot = output_wavefront

        if self.view_type > 0:
            self.initializeTabs()
            self.do_plot_results(80.0)
        else:
            self.progressBarFinished()

        self.send("WofryData", WofryData(beamline=beamline, wavefront=output_wavefront))
        self.send("Trigger", TriggerIn(new_object=True))


        self.wofry_python_script.set_code(beamline.to_python_code())

        self.setStatusMessage("")
        try:
            self.print_intensities()
        except:
            pass

    #
    # overwritten methods
    #

    # overwritten method for specific built-in propagator
    def create_propagation_setting_tab(self):
        self.tab_pro = oasysgui.createTabPage(self.tabs_setting, "Propagation Setting")
        self.zoom_box = oasysgui.widgetBox(self.tab_pro, "", addSpace=False, orientation="vertical", height=90)
        oasysgui.lineEdit(self.zoom_box, self, "magnification_x", "Magnification Factor for interval",
                          labelWidth=260, valueType=float, orientation="horizontal")

    # overwritten methods to append profile plot
    def get_titles(self):
        titles = super().get_titles()
        titles.append("O.E. Profile")
        titles.append("Footprint")
        return titles

    def do_plot_results(self, progressBarValue=80): # OVERWRITTEN

        super().do_plot_results(progressBarValue, closeProgressBar=False)
        if not self.view_type == 0:
            if not self.wavefront_to_plot is None:

                self.progressBarSet(progressBarValue)

                x, y = self.get_optical_element().get_height_profile(self.input_data.get_wavefront())
                self.plot_data1D(x=x,
                                 y=1e6*y,
                                 progressBarValue=progressBarValue + 10,
                                 tabs_canvas_index=4,
                                 plot_canvas_index=4,
                                 calculate_fwhm=False,
                                 title=self.get_titles()[4],
                                 xtitle="Spatial Coordinate along o.e. [m]",
                                 ytitle="Profile Height [$\mu$m]")


                x, y, amplitude = self.get_optical_element().get_footprint(self.input_data.get_wavefront())
                self.plot_data1D(x=x,
                                 y=numpy.abs(amplitude)**2,
                                 progressBarValue=progressBarValue + 10,
                                 tabs_canvas_index=5,
                                 plot_canvas_index=5,
                                 calculate_fwhm=False,
                                 title=self.get_titles()[5],
                                 xtitle="Spatial Coordinate along o.e. [m]",
                                 ytitle="Intensity")



                self.plot_canvas[0].resetZoom()


                self.progressBarFinished()


if __name__ == '__main__':

    from PyQt5.QtWidgets import QApplication


    def create_wavefront():
        #
        # create input_wavefront
        #
        from wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D
        input_wavefront = GenericWavefront1D.initialize_wavefront_from_range(x_min=-0.00147, x_max=0.00147,
                                                                             number_of_points=1000)
        input_wavefront.set_photon_energy(250)
        input_wavefront.set_spherical_wave(radius=13.73, center=0, complex_amplitude=complex(1, 0))
        return input_wavefront

    app = QApplication([])
    ow = OWMirror1D()
    ow.set_input(create_wavefront())

    # ow.receive_dabam_profile(numpy.array([[1,2],[3,4]]))
    ow.propagate_wavefront()

    ow.show()
    app.exec_()
    ow.saveSettings()



import sys

from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import QRect
from PyQt5.QtWidgets import QApplication, QFileDialog
from Shadow import ShadowTools as ST
from orangewidget import gui
from oasys.widgets import gui as oasysgui, widget
from oasys.util.oasys_util import EmittingStream

from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.wofry.widgets.gui.python_script import PythonScript

class OWWOInfo(widget.OWWidget):

    name = "Info"
    description = "Display Data: Info"
    icon = "icons/info.png"
    maintainer = "M Sanchez del Rio"
    maintainer_email = "srio(@at@)esrf.eu"
    priority = 300
    category = "Data Display Tools"
    keywords = ["data", "file", "load", "read"]

    inputs = [("Input Beam", WofryData, "set_input")]

    WIDGET_WIDTH = 950
    WIDGET_HEIGHT = 650

    want_main_area=1
    want_control_area = 0

    input_data=None

    def __init__(self, show_automatic_box=True):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width()*0.05),
                               round(geom.height()*0.05),
                               round(min(geom.width()*0.98, self.WIDGET_WIDTH)),
                               round(min(geom.height()*0.95, self.WIDGET_HEIGHT))))


        gen_box = gui.widgetBox(self.mainArea, "Beamline Info", addSpace=True, orientation="horizontal")

        tabs_setting = oasysgui.tabWidget(gen_box)
        tabs_setting.setFixedHeight(self.WIDGET_HEIGHT-60)
        tabs_setting.setFixedWidth(self.WIDGET_WIDTH-60)

        tab_sys = oasysgui.createTabPage(tabs_setting, "Sys Info")
        tab_dis = oasysgui.createTabPage(tabs_setting, "Distances Summary")
        tab_scr = oasysgui.createTabPage(tabs_setting, "Python Script")
        tab_out = oasysgui.createTabPage(tabs_setting, "System Output")

        self.sysInfo = oasysgui.textArea()
        self.sysInfo.setMaximumHeight(self.WIDGET_HEIGHT-100)
        sys_box = oasysgui.widgetBox(tab_sys, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        sys_box.layout().addWidget(self.sysInfo)

        self.distancesSummary = oasysgui.textArea()
        self.distancesSummary.setMaximumHeight(self.WIDGET_HEIGHT-100)
        dist_box = oasysgui.widgetBox(tab_dis, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        dist_box.layout().addWidget(self.distancesSummary)

        self.pythonScript = oasysgui.textArea(readOnly=False)
        self.pythonScript.setMaximumHeight(self.WIDGET_HEIGHT - 300)
        script_box = gui.widgetBox(tab_scr, "Python script", addSpace=True, orientation="horizontal")
        self.wofry_python_script = PythonScript()
        self.wofry_python_script.code_area.setFixedHeight(300)
        script_box.layout().addWidget(self.wofry_python_script)



        self.wofry_output = oasysgui.textArea()
        out_box = oasysgui.widgetBox(tab_out, "System Output", addSpace=True, orientation="horizontal", height=self.WIDGET_HEIGHT - 80)
        out_box.layout().addWidget(self.wofry_output)

    def set_input(self, wofry_data):
        if not wofry_data is None:
            if isinstance(wofry_data, WofryData):
                self.input_data = wofry_data
            else:
                raise Exception("Only wofry_data allowed as input")

        self.update()

    def update(self):
        if self.input_data is None:
            return

        bl = self.input_data.get_beamline()
        self.distancesSummary.setPlainText(bl.distances())
        self.wofry_python_script.set_code(bl.to_python_code())
        self.sysInfo.setPlainText(bl.info())


    def writeStdOut(self, text):
        cursor = self.shadow_output.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertText(text)
        self.wofryoutput.setTextCursor(cursor)
        self.wofryoutput.ensureCursorVisible()


if __name__ == "__main__":
    import sys
    from PyQt5.QtWidgets import QApplication

    a = QApplication(sys.argv)
    ow = OWWOInfo()


    ow.show()
    a.exec_()
    ow.saveSettings()


from orangecontrib.wofry.widgets.gui.ow_optical_element_1d import OWWOOpticalElement1D
from syned.beamline.optical_elements.ideal_elements.screen import Screen

from orangewidget.settings import Setting

from wofryimpl.beamline.optical_elements.ideal_elements.screen import WOScreen1D

#################
import numpy, sys

from PyQt5.QtGui import QPalette, QColor, QFont
from PyQt5.QtWidgets import QMessageBox

from orangewidget import gui
from orangewidget import widget
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.widgets.gui import ConfirmDialog
from oasys.util.oasys_util import EmittingStream, TriggerIn

from syned.widget.widget_decorator import WidgetDecorator
from syned.beamline.element_coordinates import ElementCoordinates
from syned.beamline.beamline_element import BeamlineElement
from syned.beamline.shape import *

from wofry.propagator.propagator import PropagationManager, PropagationElements, PropagationParameters
from wofryimpl.propagator.propagators1D.fresnel import Fresnel1D
from wofryimpl.propagator.propagators1D.fresnel_convolution import FresnelConvolution1D
from wofryimpl.propagator.propagators1D.fraunhofer import Fraunhofer1D
from wofryimpl.propagator.propagators1D.integral import Integral1D
from wofryimpl.propagator.propagators1D.fresnel_zoom import FresnelZoom1D
from wofryimpl.propagator.propagators1D.fresnel_zoom_scaling_theorem import FresnelZoomScaling1D

from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget
#################

from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget
from oasys.util.oasys_util import get_fwhm

class OWWOScanningScreen1D(OWWOOpticalElement1D):

    name = "Scanning Screen 1D"
    description = "Wofry: Scanning Screen 1D"
    # icon = "icons/scanningscreen1d.png"
    icon = "icons/caustic.png"
    priority = 200

    q_min = Setting(1.0)
    q_max = Setting(2.0)
    q_points = Setting(2)
    file_flag = Setting(0)
    file_name = Setting("tmp.h5")

    def __init__(self,is_automatic=True, show_view_options=True, show_script_tab=True):
        WofryWidget.__init__(self,is_automatic=is_automatic, show_view_options=show_view_options, show_script_tab=show_script_tab)

        self.runaction = widget.OWAction("Propagate Wavefront", self)
        self.runaction.triggered.connect(self.propagate_wavefront)
        self.addAction(self.runaction)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Propagate Wavefront", callback=self.propagate_wavefront)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        button = gui.button(button_box, self, "Reset Fields", callback=self.callResetSettings)
        font = QFont(button.font())
        font.setItalic(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Red'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)
        button.setFixedWidth(150)

        gui.separator(self.controlArea)

        self.tabs_setting = oasysgui.tabWidget(self.controlArea)
        self.tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT)
        self.tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)


        self.tab_bas = oasysgui.createTabPage(self.tabs_setting, "O.E. Setting")


        oasysgui.lineEdit(self.tab_bas, self, "oe_name", "O.E. Name", labelWidth=260, valueType=str, orientation="horizontal")

        self.coordinates_box = oasysgui.widgetBox(self.tab_bas, "Coordinates", addSpace=True, orientation="vertical")

        tmp = oasysgui.lineEdit(self.coordinates_box, self, "p", "Distance from previous Continuation Plane [m]", labelWidth=280, valueType=float, orientation="horizontal")
        tmp.setToolTip("p")
        tmp = oasysgui.lineEdit(self.coordinates_box, self, "q_min", "Min distance to next Continuation Plane [m]", labelWidth=280, valueType=float, orientation="horizontal")
        tmp.setToolTip("q_min")
        tmp = oasysgui.lineEdit(self.coordinates_box, self, "q_max", "Max distance to next Continuation Plane [m]", labelWidth=280, valueType=float, orientation="horizontal")
        tmp.setToolTip("q_max")
        tmp = oasysgui.lineEdit(self.coordinates_box, self, "q_points", "scanning points in p", labelWidth=280, valueType=int, orientation="horizontal")
        tmp.setToolTip("q_points")


        gui.comboBox(self.coordinates_box, self, "file_flag", label="Dump file", labelWidth=260,
                     items=["No","Yes"],
                     callback=self.set_file_flag,
                     sendSelectedValue=False, orientation="horizontal")

        self.file_box = oasysgui.widgetBox(self.coordinates_box, "", addSpace=False, orientation="horizontal")
        tmp = oasysgui.lineEdit(self.file_box, self, "file_name", "File name [.h5]", labelWidth=280, valueType=int, orientation="horizontal")
        tmp.setToolTip("file_name")


        self.draw_specific_box()

        self.create_propagation_setting_tab()

        self.set_file_flag()

    def set_file_flag(self):
        self.file_box.setVisible(self.file_flag == 0)
        self.file_box.setVisible(self.file_flag == 1)

    def get_optical_element(self):
        return WOScreen1D(name=self.oe_name)

    def check_syned_instance(self, optical_element):
        if not isinstance(optical_element, Screen):
            raise Exception("Syned Data not correct: Optical Element is not a Screen")

    def propagate_wavefront(self):

        qs = numpy.linspace(self.q_min, self.q_max, self.q_points)

        view_type_old = self.view_type
        self.view_type = 0

        wavefronts = []
        peak = numpy.zeros(self.q_points)
        fwhm = numpy.zeros(self.q_points)
        integral = numpy.zeros(self.q_points)

        for i,q in enumerate(qs):
            self.q = q
            super().propagate_wavefront()
            wavefronts.append(self.wavefront_to_plot)
            if i == 0:
                intensities = numpy.zeros((self.q_points,self.wavefront_to_plot.get_abscissas().size))
                x = self.wavefront_to_plot.get_abscissas()

            active_profile = self.wavefront_to_plot.get_intensity()

            intensities[i,:] = active_profile

            peak[i] = numpy.max(active_profile)
            try:
                fwhm[i], _, _ = get_fwhm(active_profile, x)
                print("fwhm: ", i, fwhm[i])
            except:
                fwhm[i] = 0.0
                print("bad fwhm: ", i, fwhm[i])

        if self.file_flag:
            from srxraylib.util.h5_simple_writer import H5SimpleWriter
            h5w = H5SimpleWriter.initialize_file(self.file_name, overwrite=1)
            for i, q in enumerate(qs):
                if i == 0:
                    # from srxraylib.util.h5_simple_writer import H5SimpleWriter
                    # h5w = H5SimpleWriter.initialize_file(self.file_name, creator="h5_basic_writer.py")


                    wavefronts[i].save_h5_file(self.file_name, subgroupname="wfr%03d" % i,
                                                        intensity=True, phase=False, overwrite=True, verbose=False)
                else:
                    wavefronts[i].save_h5_file(self.file_name, subgroupname="wfr%03d" % i,
                                                            intensity=True, phase=False, overwrite=False, verbose=False)


            h5w.add_dataset(qs, peak, dataset_name="peak", entry_name=None, title_x="q [m]", title_y="peak")
            h5w.add_dataset(qs, fwhm*1e6, dataset_name="fwhm", entry_name=None, title_x="q [m]", title_y="fwhm [$\mu$m]")
            h5w.add_image(intensities, image_x=qs,image_y=x*1e6,image_name="scan",entry_name=None,title_x="q [m]",
                          title_y="Spatial Coordinate [$\mu$m]")
            print("\nFile %s written to disk." % self.file_name)

        print("\nNumber of wavefronts generated: ", len(wavefronts))

        self.view_type = view_type_old
        if self.view_type > 0:
            self.initializeTabs()
            self.do_plot_results(closeProgressBar=False)
            #


            self.plot_data1D(x=qs,
                             y=peak,
                             progressBarValue=97,
                             tabs_canvas_index=4,
                             plot_canvas_index=4,
                             calculate_fwhm=False,
                             title="peak evolution q_points=%s" % self.q_points,
                             xtitle="q [m]",
                             ytitle="Peak Intensity")

            self.plot_data1D(x=qs,
                             y=1e6*fwhm,
                             progressBarValue=98,
                             tabs_canvas_index=5,
                             plot_canvas_index=5,
                             calculate_fwhm=False,
                             title="FWHM evolution q_points=%s" % self.q_points,
                             xtitle="q [m]",
                             ytitle="FWHM [$\mu$m]")

            self.plot_data2D(intensities, qs, 1e6*wavefronts[-1].get_abscissas(),
                             progressBarValue=99,
                             tabs_canvas_index=6,
                             plot_canvas_index=6,
                             title="q_points=%s" % self.q_points,
                             xtitle="q [m]",
                             ytitle="Spatial Coordinate [$\mu$m]",
                             )
            self.progressBarFinished()

    def initializeTabs(self):
        super().initializeTabs()

        self.tab.append(gui.createTabPage(self.tabs, "Peak"))
        self.plot_canvas.append(None)
        self.tab[-1].setFixedHeight(self.IMAGE_HEIGHT)
        self.tab[-1].setFixedWidth(self.IMAGE_WIDTH)

        self.tab.append(gui.createTabPage(self.tabs, "FWHM"))
        self.plot_canvas.append(None)
        self.tab[-1].setFixedHeight(self.IMAGE_HEIGHT)
        self.tab[-1].setFixedWidth(self.IMAGE_WIDTH)

        self.tab.append(gui.createTabPage(self.tabs, "Scan"))
        self.plot_canvas.append(None)
        self.tab[-1].setFixedHeight(self.IMAGE_HEIGHT)
        self.tab[-1].setFixedWidth(self.IMAGE_WIDTH)


if __name__ == "__main__":
    import sys
    from PyQt5.QtWidgets import QApplication

    def get_example_wofry_data():
        from wofryimpl.propagator.light_source import WOLightSource
        from wofryimpl.beamline.beamline import WOBeamline
        from orangecontrib.wofry.util.wofry_objects import WofryData

        light_source = WOLightSource(dimension=1,
                                     initialize_from=0,
                                     range_from_h=-0.001,
                                     range_to_h=0.001,
                                     number_of_points_h=500,
                                     energy=10000.0,
                                     )

        return WofryData(wavefront=light_source.get_wavefront(),
                           beamline=WOBeamline(light_source=light_source))

    a = QApplication(sys.argv)
    ow = OWWOScanningScreen1D()
    ow.set_input(get_example_wofry_data())

    ow.show()
    a.exec_()
    ow.saveSettings()




from oasys.widgets import gui as oasysgui
from silx.gui.plot import Plot2D


def plot_data2D(data2D, dataX, dataY, title="", xtitle="", ytitle=""):
    origin = (dataX[0], dataY[0])
    scale = (dataX[1] - dataX[0], dataY[1] - dataY[0])

    colormap = {"name": "temperature", "normalization": "linear", "autoscale": True, "vmin": 0, "vmax": 0,
                "colors": 256}

    plot_canvas = Plot2D()
    plot_canvas.resetZoom()
    plot_canvas.setXAxisAutoScale(True)
    plot_canvas.setYAxisAutoScale(True)
    plot_canvas.setGraphGrid(False)
    plot_canvas.setKeepDataAspectRatio(True)
    plot_canvas.yAxisInvertedAction.setVisible(False)
    plot_canvas.setXAxisLogarithmic(False)
    plot_canvas.setYAxisLogarithmic(False)
    plot_canvas.getMaskAction().setVisible(False)
    plot_canvas.getRoiAction().setVisible(False)
    plot_canvas.getColormapAction().setVisible(True)
    plot_canvas.setKeepDataAspectRatio(False)
    #
    #
    #
    plot_canvas.addImage(data2D.T,
                         legend="",
                         scale=scale,
                         origin=origin,
                         colormap=colormap,
                         replace=True)
    plot_canvas.setActiveImage("")
    plot_canvas.setGraphXLabel(xtitle)
    plot_canvas.setGraphYLabel(ytitle)
    plot_canvas.setGraphTitle(title)
    # try:
    #     image_box.layout().removeItem(image_box.layout().itemAt(0))
    # except:
    #     pass
    # image_box.layout().addWidget(plot_canvas)
    return plot_canvas


def plot_data1D(x, y, x2=None, y2=None,
                title="", xtitle="", ytitle="",
                log_x=False, log_y=False, color='blue', replace=True, control=False,
                xrange=None, yrange=None, symbol=''):



    plot_widget_id = oasysgui.plotWindow(parent=None,
                                         backend=None,
                                         resetzoom=True,
                                         autoScale=False,
                                         logScale=True,
                                         grid=True,
                                         curveStyle=True,
                                         colormap=False,
                                         aspectRatio=False,
                                         yInverted=False,
                                         copy=True,
                                         save=True,
                                         print_=True,
                                         control=control,
                                         position=True,
                                         roi=False,
                                         mask=False,
                                         fit=False)


    plot_widget_id.setDefaultPlotLines(True)
    plot_widget_id.setActiveCurveColor(color='blue')
    plot_widget_id.setGraphXLabel(xtitle)
    plot_widget_id.setGraphYLabel(ytitle)


    plot_widget_id.addCurve(x, y, title, symbol=symbol, color=color, xlabel=xtitle, ylabel=ytitle, replace=replace)  # '+', '^', ','
    if x2 is not None and y2 is not None:
        plot_widget_id.addCurve(x2, y2)


    if not xtitle is None: plot_widget_id.setGraphXLabel(xtitle)
    if not ytitle is None: plot_widget_id.setGraphYLabel(ytitle)
    if not title is None:  plot_widget_id.setGraphTitle(title)

    plot_widget_id.resetZoom()
    plot_widget_id.replot()
    plot_widget_id.setActiveCurve(title)


    plot_widget_id.setXAxisLogarithmic(log_x)
    plot_widget_id.setYAxisLogarithmic(log_y)

    print(">>>>>>>>>>>>>>>>>>>>>>>", yrange)
    if xrange is not None:
        plot_widget_id.setGraphXLimits(xrange[0] ,xrange[1])
    if yrange is not None:
        plot_widget_id.setGraphYLimits(yrange[0] ,yrange[1])

    print(">>>>>>>>>>>>>>>>>>>>>>>",yrange)
    # if min(y) < 0:
    #     if log_y:
    #         plot_widget_id.setGraphYLimits(min(y ) *1.2, max(y ) *1.2)
    #     else:
    #         plot_widget_id.setGraphYLimits(min(y ) *1.01, max(y ) *1.01)
    # else:
    #     if log_y:
    #         plot_widget_id.setGraphYLimits(min(y), max(y ) *1.2)
    #     else:
    #         plot_widget_id.setGraphYLimits(min(y ) *0.99, max(y ) *1.01)

    return plot_widget_id




import numpy
import sys

from PyQt5.QtGui import QPalette, QColor, QFont
from PyQt5.QtWidgets import QMessageBox

from orangewidget import gui
from orangewidget import widget
from orangewidget.settings import Setting

from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.util.oasys_util import TriggerIn, TriggerOut, EmittingStream

from syned.storage_ring.magnetic_structures.undulator import Undulator
from syned.beamline.beamline import Beamline

from wofryimpl.propagator.light_source import WOLightSource
from wofryimpl.beamline.beamline import WOBeamline

from orangecontrib.wofry.util.wofry_objects import WofryData
from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget

import scipy.constants as codata

from wofryimpl.propagator.util.undulator_coherent_mode_decomposition_1d import UndulatorCoherentModeDecomposition1D
from syned.storage_ring.electron_beam import ElectronBeam
from syned.storage_ring.magnetic_structures.undulator import Undulator
from orangecontrib.esrf.wofry.util.light_source import WOLightSourceCMD


class OWUndulatorCoherentModeDecomposition1D(WofryWidget):

    name = "Undulator Coherent Mode Decomposition 1D"
    id = "UndulatorCMD1D"
    description = "Undulator Coherent Mode Decomposition 1D"
    icon = "icons/undulator_cmd_1d.png"
    priority = 0.4

    category = "Wofry Wavefront Propagation"
    keywords = ["data", "file", "load", "read"]

    inputs = [
                ("SynedData", Beamline, "receive_syned_data"),
                ("Trigger", TriggerOut, "receive_trigger_signal"),
                ]
    outputs = [
               {"name":"WofryData",
                "type":WofryData,
                "doc":"WofryData",
                "id":"WofryData"}
                ]

    number_of_points = Setting(200)
    initialize_from  = Setting(0)
    range_from       = Setting(-0.000125)
    range_to         = Setting( 0.000125)
    steps_start      = Setting(-0.00005)
    steps_step       = Setting(1e-7)


    sigma_h = Setting(3.01836e-05)
    sigma_v = Setting(3.63641e-06)
    sigma_divergence_h = Setting(4.36821e-06)
    sigma_divergence_v = Setting(1.37498e-06)

    photon_energy = Setting(10000.0)

    period_length = Setting(0.020)
    number_of_periods = Setting(100)
    K_vertical = Setting(1.19)
    electron_energy_in_GeV = Setting(6.0)
    ring_current = Setting(0.2)
    # sigma_h sigma_divergence_h sigma_v sigma_divergence_v




    flag_gsm = Setting(0)
    scan_direction_flag = Setting(0)
    mode_index = Setting(0)

    spectral_density_threshold = Setting(0.99)
    correction_factor = Setting(1.0)

    # to store calculations
    coherent_mode_decomposition = None
    coherent_mode_decomposition_results = None

    def __init__(self):

        super().__init__(is_automatic=False, show_view_options=True, show_script_tab=True)

        self.runaction = widget.OWAction("Generate Wavefront", self)
        self.runaction.triggered.connect(self.calculate_and_send_mode)
        self.addAction(self.runaction)


        gui.separator(self.controlArea)
        gui.separator(self.controlArea)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Calculate and Send mode", callback=self.calculate_and_send_mode)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        gui.separator(self.controlArea)

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT + 50)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        self.tab_settings = oasysgui.createTabPage(tabs_setting, "Settings")
        self.tab_lightsource = oasysgui.createTabPage(tabs_setting, "Light Source")

        #
        # Settings
        #
        box_space = oasysgui.widgetBox(self.tab_settings, "Sampling coordinates", addSpace=False, orientation="vertical")

        oasysgui.lineEdit(box_space, self, "number_of_points", "Number of Points",
                          labelWidth=300, tooltip="number_of_points",
                          valueType=int, orientation="horizontal")

        gui.comboBox(box_space, self, "initialize_from", label="Space Initialization",
                     labelWidth=350,
                     items=["From Range", "From Steps"],
                     callback=self.set_Initialization,
                     sendSelectedValue=False, orientation="horizontal")

        self.initialization_box_1 = oasysgui.widgetBox(box_space, "", addSpace=False, orientation="vertical")

        oasysgui.lineEdit(self.initialization_box_1, self, "range_from", "From  [m]",
                          labelWidth=200, tooltip="range_from",
                          valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.initialization_box_1, self, "range_to", "To [m]",
                          labelWidth=200, tooltip="range_to",
                          valueType=float, orientation="horizontal")

        self.initialization_box_2 = oasysgui.widgetBox(box_space, "", addSpace=False, orientation="vertical")

        oasysgui.lineEdit(self.initialization_box_2, self, "steps_start", "Start [m]",
                          labelWidth=300, tooltip="steps_start",
                          valueType=float, orientation="horizontal")

        oasysgui.lineEdit(self.initialization_box_2, self, "steps_step", "Step [m]",
                          labelWidth=300, tooltip="steps_step",
                          valueType=float, orientation="horizontal")

        self.set_Initialization()


        left_box_3 = oasysgui.widgetBox(self.tab_settings, "Setting", addSpace=True, orientation="vertical")

        left_box_33 = oasysgui.widgetBox(left_box_3, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(left_box_33, self, "photon_energy", "Photon Energy [eV]",
                          labelWidth=200, tooltip="photon_energy",
                          valueType=float, orientation="horizontal")
        gui.button(left_box_33, self, "set from K", callback=self.set_photon_energy, width=80)


        gui.comboBox(left_box_3, self, "flag_gsm", label="Decomposition", labelWidth=120,
                     items=["Undulator Coherent Mode Decomposition",
                            "Gaussian Shell-model approximation",
                            ],
                     callback=self.set_visible,
                     sendSelectedValue=False, orientation="horizontal")

        gui.comboBox(left_box_3, self, "scan_direction_flag", label="Direction", labelWidth=350,
                     items=["Horizontal",
                            "Vertical"
                            ],
                     callback=self.set_visible,
                     sendSelectedValue=False, orientation="horizontal")

        left_box_3 = oasysgui.widgetBox(self.tab_settings, "Send Wavefront", addSpace=True, orientation="vertical")
        self.mode_index_box = oasysgui.widgetBox(left_box_3, "", addSpace=True, orientation="vertical", )

        left_box_5 = oasysgui.widgetBox(self.mode_index_box, "", addSpace=True, orientation="horizontal", )
        tmp = oasysgui.lineEdit(left_box_5, self, "mode_index", "Send mode",
                        labelWidth=200, valueType=int, tooltip = "mode_index",
                        orientation="horizontal", callback=self.send_mode)

        gui.button(left_box_5, self, "+1", callback=self.increase_mode_index, width=30)
        gui.button(left_box_5, self, "-1", callback=self.decrease_mode_index, width=30)
        gui.button(left_box_5, self,  "0", callback=self.reset_mode_index, width=30)

        #
        # Light Source
        #

        storage_ring_box = oasysgui.widgetBox(self.tab_lightsource, "Storage Ring",
                                            addSpace=True, orientation="vertical")

        oasysgui.lineEdit(storage_ring_box, self, "electron_energy_in_GeV", "Energy [GeV]",  labelWidth=260, valueType=float, orientation="horizontal", callback=self.update)
        oasysgui.lineEdit(storage_ring_box, self, "ring_current", "Ring Current [A]",        labelWidth=260, valueType=float, orientation="horizontal", callback=self.update)




        self.emittances_box_h = oasysgui.widgetBox(self.tab_lightsource, "Electron Horizontal beam sizes",
                                            addSpace=True, orientation="vertical")
        self.emittances_box_v = oasysgui.widgetBox(self.tab_lightsource, "Electron Vertical beam sizes",
                                            addSpace=True, orientation="vertical")


        self.le_sigma_h = oasysgui.lineEdit(self.emittances_box_h, self, "sigma_h", "Size RMS H [m]",
                            labelWidth=250, tooltip="sigma_h",
                            valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.emittances_box_h, self, "sigma_divergence_h", "Divergence RMS H [rad]",
                            labelWidth=250, tooltip="sigma_divergence_h",
                            valueType=float, orientation="horizontal")


        self.le_sigma_v = oasysgui.lineEdit(self.emittances_box_v, self, "sigma_v", "Size RMS V [m]",
                            labelWidth=250, tooltip="sigma_v",
                            valueType=float, orientation="horizontal")
        oasysgui.lineEdit(self.emittances_box_v, self, "sigma_divergence_v", "Divergence RMS V [rad]",
                            labelWidth=250, tooltip="sigma_divergence_v",
                            valueType=float, orientation="horizontal")





        # oasysgui.lineEdit(self.left_box_2_2, self, "electron_beam_size_h",       "Horizontal Beam Size \u03c3x [m]",          labelWidth=260, valueType=float, orientation="horizontal",  callback=self.update)
        # oasysgui.lineEdit(self.left_box_2_2, self, "electron_beam_size_v",       "Vertical Beam Size \u03c3y [m]",            labelWidth=260, valueType=float, orientation="horizontal",  callback=self.update)
        # oasysgui.lineEdit(self.left_box_2_2, self, "electron_beam_divergence_h", "Horizontal Beam Divergence \u03c3'x [rad]", labelWidth=260, valueType=float, orientation="horizontal",  callback=self.update)
        # oasysgui.lineEdit(self.left_box_2_2, self, "electron_beam_divergence_v", "Vertical Beam Divergence \u03c3'y [rad]",   labelWidth=260, valueType=float, orientation="horizontal",  callback=self.update)

        ###################


        left_box_1 = oasysgui.widgetBox(self.tab_lightsource, "ID Parameters", addSpace=True, orientation="vertical")

        oasysgui.lineEdit(left_box_1, self, "period_length", "Period Length [m]", labelWidth=260,
                          valueType=float, orientation="horizontal", callback=self.update)
        oasysgui.lineEdit(left_box_1, self, "number_of_periods", "Number of Periods", labelWidth=260,
                          valueType=float, orientation="horizontal", callback=self.update)

        oasysgui.lineEdit(left_box_1, self, "K_vertical", "Vertical K", labelWidth=260,
                          valueType=float, orientation="horizontal")


        self.set_visible()


    def set_visible(self):
        self.emittances_box_h.setVisible(self.scan_direction_flag == 0)
        self.emittances_box_v.setVisible(self.scan_direction_flag == 1)

    def increase_mode_index(self):
        self.mode_index += 1
        if self.coherent_mode_decomposition is None:
            self.calculate()
        else:
            self.send_mode()

    def decrease_mode_index(self):
        self.mode_index -= 1
        if self.mode_index < 0: self.mode_index = 0
        if self.coherent_mode_decomposition is None:
            self.calculate()
        else:
            self.send_mode()

    def reset_mode_index(self):
        self.mode_index = 0
        if self.coherent_mode_decomposition is None:
            self.calculate()
        else:
            self.send_mode()

    def set_Initialization(self):
        self.initialization_box_1.setVisible(self.initialize_from == 0)
        self.initialization_box_2.setVisible(self.initialize_from == 1)


    def initializeTabs(self):
        size = len(self.tab)
        indexes = range(0, size)

        current_index = self.tabs.currentIndex()

        for index in indexes:
            self.tabs.removeTab(size-1-index)

        self.titles = ["Emission size",
                       "Far field emission",
                       "Cross Spectral Density",
                       "Cumulated occupation",
                       "Eigenfunctions",
                       "Spectral Density",
                       "Sent mode"]
        self.tab = []
        self.plot_canvas = []

        for index in range(0, len(self.titles)):
            self.tab.append(gui.createTabPage(self.tabs, self.titles[index]))
            self.plot_canvas.append(None)

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)

        if current_index < 0:
            current_index = len(self.titles) - 1
        self.tabs.setCurrentIndex(current_index)

    def set_photon_energy(self):
        ebeam = ElectronBeam(energy_in_GeV=self.electron_energy_in_GeV,
                             current=self.ring_current)
        su = Undulator.initialize_as_vertical_undulator(K=self.K_vertical,
                                                        period_length=self.period_length,
                                                        periods_number=self.number_of_periods)
        self.photon_energy = numpy.round(su.resonance_energy(ebeam.gamma(), harmonic=1.0), 3)

    def check_fields(self):
        congruence.checkStrictlyPositiveNumber(self.photon_energy, "Photon Energy")

        if self.initialize_from == 0:
            congruence.checkGreaterThan(self.range_to, self.range_from, "Range To", "Range From")
        else:
            congruence.checkStrictlyPositiveNumber(self.steps_step, "Step")

        congruence.checkStrictlyPositiveNumber(self.number_of_points, "Number of Points")

        congruence.checkNumber(self.mode_index, "Mode index")

        congruence.checkStrictlyPositiveNumber(self.spectral_density_threshold, "Threshold")

        congruence.checkStrictlyPositiveNumber(self.correction_factor, "Correction factor for SigmaI")


    def receive_syned_data(self, data):
        if not data is None:
            if isinstance(data, Beamline):
                if not data._light_source is None:
                    if isinstance(data._light_source._magnetic_structure, Undulator):
                        light_source = data._light_source
                        ebeam = light_source.get_electron_beam()
                        und = light_source.get_magnetic_structure()

                        x, xp, y, yp = ebeam.get_sigmas_all()
                        self.sigma_h = x
                        self.sigma_v = y
                        self.sigma_divergence_h = xp
                        self.sigma_divergence_v = yp
                        self.electron_energy_in_GeV = ebeam.energy()
                        self.ring_current = ebeam.current()

                        self.number_of_periods = und.number_of_periods()
                        self.period_length = und.period_length()
                        self.photon_energy =  round(und.resonance_energy(ebeam.gamma()), 3)
                        self.K_vertical = und.K_vertical()


                    else:
                        raise ValueError("Syned light source not congruent")
                else:
                    raise ValueError("Syned data not correct: light source not present")
            else:
                raise ValueError("Syned data not correct")

    def receive_trigger_signal(self, trigger):

        if trigger and trigger.new_object == True:
            if trigger.has_additional_parameter("variable_name"):
                variable_name = trigger.get_additional_parameter("variable_name").strip()
                variable_display_name = trigger.get_additional_parameter("variable_display_name").strip()
                variable_value = trigger.get_additional_parameter("variable_value")
                variable_um = trigger.get_additional_parameter("variable_um")

                if "," in variable_name:
                    variable_names = variable_name.split(",")

                    for variable_name in variable_names:
                        setattr(self, variable_name.strip(), variable_value)
                else:
                    setattr(self, variable_name, variable_value)

                self.send_mode()

    def get_light_source(self):
        return WOLightSourceCMD(name="name", undulator_coherent_mode_decomposition_1d=self.coherent_mode_decomposition)

    def generate(self):
        pass

    def calculate_and_send_mode(self):
        self.calculate()
        self.send_mode()

    def calculate(self):

        self.wofry_output.setText("")

        sys.stdout = EmittingStream(textWritten=self.writeStdOut)


        self.progressBarInit()
        self.progressBarSet(15)

        self.check_fields()


        if self.scan_direction_flag == 0:
            scan_direction = "H"
            sigmaxx=self.sigma_h
            sigmaxpxp=self.sigma_divergence_h
        else:
            scan_direction = "V"
            sigmaxx=self.sigma_v
            sigmaxpxp=self.sigma_divergence_v

        if self.flag_gsm == 0:
            useGSMapproximation = False
        elif self.flag_gsm == 1:
            useGSMapproximation = True

        # main calculation
        self.coherent_mode_decomposition = UndulatorCoherentModeDecomposition1D(
            electron_energy=self.electron_energy_in_GeV,
            electron_current=self.ring_current,
            undulator_period=self.period_length,
            undulator_nperiods=self.number_of_periods,
            K=self.K_vertical,
            photon_energy=self.photon_energy,
            abscissas_interval=self.range_to - self.range_from,
            number_of_points=self.number_of_points,
            distance_to_screen=100.0,
            scan_direction=scan_direction,
            sigmaxx=sigmaxx,
            sigmaxpxp=sigmaxpxp,
            useGSMapproximation=useGSMapproximation)
        # make calculation
        self.coherent_mode_decomposition_results = self.coherent_mode_decomposition.calculate()

        if self.view_type != 0:
            self.initializeTabs()
            self.plot_results()
        else:
            self.progressBarFinished()

        try:
            beamline = WOBeamline(light_source=self.get_light_source())
            self.wofry_python_script.set_code(beamline.to_python_code())
        except:
            pass

        # self.send_mode()

    def send_mode(self):

        if self.coherent_mode_decomposition is None:
            self.calculate()

        if self.view_type != 0:
            self.do_plot_send_mode()

        beamline = WOBeamline(light_source=self.get_light_source())
        print(">>> sending mode: ", int(self.mode_index))
        self.send("WofryData", WofryData(
            wavefront=self.coherent_mode_decomposition.get_eigenvector_wavefront(int(self.mode_index)),
            beamline=beamline))

    def do_plot_send_mode(self):
        if not self.coherent_mode_decomposition is None:

            #
            # plot mode to send
            #
            abscissas = self.coherent_mode_decomposition_results["abscissas"]
            wf = self.coherent_mode_decomposition.get_eigenvector_wavefront(self.mode_index)

            xtitle = "Photon energy [keV]"
            ytitle = "wavefront intensity"

            self.plot_data1D(1e6 * abscissas,
                             wf.get_intensity(),
                             progressBarValue=90.0,
                             tabs_canvas_index=6,
                             plot_canvas_index=6,
                             title=self.titles[6],
                             xtitle="Spatial Coordinate [$\mu$m]",
                             ytitle="Intensity",
                             calculate_fwhm=True)

            self.progressBarFinished()

    def do_plot_results(self, progressBarValue):
        if not self.coherent_mode_decomposition is None:

            self.progressBarSet(progressBarValue)

            #
            # plot emission size
            #
            if self.flag_gsm:
                abscissas = self.coherent_mode_decomposition.abscissas
                indices = numpy.arange(abscissas.size)
                intensity = self.coherent_mode_decomposition.CSD[indices,indices]
            else:
                abscissas = self.coherent_mode_decomposition.abscissas
                intensity = self.coherent_mode_decomposition.output_wavefront.get_intensity()
            self.plot_data1D(1e6 * abscissas,
                             intensity,
                             progressBarValue=progressBarValue,
                             tabs_canvas_index=0,
                             plot_canvas_index=0,
                             title=self.titles[0] + " (backpropagated)",
                             xtitle="Spatial Coordinate [$\mu$m]",
                             ytitle="Intensity",
                             calculate_fwhm=True)

            #
            # plot emission size
            #
            if self.flag_gsm:
                pass
            else:
                wfr = self.coherent_mode_decomposition.far_field_wavefront
                abscissas = wfr.get_abscissas()
                intensity = wfr.get_intensity()
                self.plot_data1D(1e6 * abscissas,
                                 intensity,
                                 progressBarValue=progressBarValue,
                                 tabs_canvas_index=1,
                                 plot_canvas_index=1,
                                 title=self.titles[1],
                                 xtitle="Spatial Coordinate [$\mu$m]",
                                 ytitle="Intensity at far field (%g m)" % self.coherent_mode_decomposition.distance_to_screen,
                                 calculate_fwhm=True)


            #
            # plot CSD
            #
            abscissas = self.coherent_mode_decomposition_results["abscissas"]
            CSD = self.coherent_mode_decomposition_results["CSD"]
            self.plot_data2D(numpy.abs(CSD),
                             1e6 * abscissas,
                             1e6 * abscissas,
                             progressBarValue, 2, 1,
                             title=self.titles[2],
                             xtitle="Spatial Coordinate x1 [$\mu$m]",
                             ytitle="Spatial Coordinate x2 [$\mu$m]")

            #
            # plot cumulated occupation
            #
            eigenvalues  = self.coherent_mode_decomposition_results["eigenvalues"]
            eigenvectors = self.coherent_mode_decomposition_results["eigenvectors"]


            nmodes = self.number_of_points
            x = numpy.arange(eigenvalues.size)
            occupation = eigenvalues[0:nmodes] / (eigenvalues.sum())
            cumulated_occupation = numpy.cumsum(occupation)

            self.plot_data1D(x,
                             cumulated_occupation,
                             progressBarValue=progressBarValue,
                             tabs_canvas_index=3,
                             plot_canvas_index=3,
                             title=self.titles[3],
                             xtitle="mode index",
                             ytitle="Cumulated occupation",
                             calculate_fwhm=False)


            #
            # plot eigenfunctions
            #
            xtitle = "Photon energy [keV]"
            ytitle = "eigenfunction"
            colors = ['green', 'black', 'red', 'brown', 'orange', 'pink']
            y_list = []
            for i in range(6):
                y_list.append(numpy.real(eigenvectors[i,:]).copy())
            ytitles = []
            for i in range(6):
                ytitles.append("eigenvalue %d" % i)

            self.plot_multi_data1D(1e6*abscissas,
                             y_list,
                             progressBarValue=progressBarValue,
                             tabs_canvas_index=4,
                             plot_canvas_index=4,
                             title=self.titles[4],
                             xtitle="x [um]",
                             ytitles=ytitles,
                             colors=colors,
                             yrange=[-eigenvectors.max(), eigenvectors.max()])

            #
            # plot spectral density
            #
            xtitle = "Photon energy [keV]"
            ytitle = "spectral density"
            colors = ['green', 'black', 'red', 'brown', 'orange', 'pink']

            SD = numpy.zeros_like(abscissas)
            for i in range(SD.size):
                SD[i] = numpy.real(CSD[i, i])

            # restore spectral density from modes
            y = numpy.zeros_like(abscissas, dtype=complex)
            nmodes = abscissas.size
            for i in range(nmodes):
                y += eigenvalues[i] * numpy.conjugate(eigenvectors[i, :]) * eigenvectors[i, :]

            self.plot_multi_data1D(1e6 * abscissas,
                             [SD,numpy.real(y)],
                             progressBarValue=progressBarValue,
                             tabs_canvas_index=5,
                             plot_canvas_index=5,
                             title=self.titles[5],
                             xtitle="x [um]",
                             ytitles=["SD from CSD","SD from modes"],
                             colors=colors)


            #
            # plot mode to be sent and close progress bar
            #
            self.do_plot_send_mode()



if __name__ == "__main__":
    import sys
    from PyQt5.QtWidgets import QApplication

    a = QApplication(sys.argv)
    ow = OWUndulatorCoherentModeDecomposition1D()

    ow.show()
    a.exec_()
    ow.saveSettings()



import numpy

from wofry.beamline.decorators import LightSourceDecorator
from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D
from wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D
from syned.storage_ring.light_source import LightSource


class WOLightSource(LightSource, LightSourceDecorator):
    def __init__(self,
                 name                = "Undefined",
                 electron_beam       = None,
                 magnetic_structure  = None,
                 dimension           = 1,
                 initialize_from     = 0,
                 range_from_h        = -1e-6,
                 range_to_h          = 1e-6,
                 range_from_v        = -1e-6,
                 range_to_v          = 1e-6,
                 steps_start_h       = -1e-6,
                 steps_step_h        = 1e-8,
                 steps_start_v       = -1e-6,
                 steps_step_v        = 1e-8,
                 number_of_points_h  = 200,
                 number_of_points_v  = 100,
                 energy              = 10000.0,
                 sigma_h             = 1e-7,
                 sigma_v             = 1e-7,
                 amplitude           = 1.0,
                 kind_of_wave        = 0, #  0=plane, 1=spherical, 2=Gaussian, 3=Gaussian-Hermite
                 n_h                 = 0,
                 n_v                 = 0,
                 beta_h              = 1.0,
                 beta_v              = 1.0, ############
                 units               = 0,
                 wavelength          = 1e-10,
                 initialize_amplitude = 0,
                 complex_amplitude_re = 1.0,
                 complex_amplitude_im = 0.0,
                 phase                = 0.0,
                 radius               = 100.0,
                 center               = 0.0,
                 inclination          = 0.0,
                 gaussian_shift       = 0.0,
                 add_random_phase     = 0,
                 ):

        super().__init__(name=name, electron_beam=electron_beam, magnetic_structure=magnetic_structure)

        self._dimension =  dimension
        self._initialize_from =  initialize_from
        self._range_from_h =  range_from_h
        self._range_to_h =  range_to_h
        self._range_from_v =  range_from_v
        self._range_to_v =  range_to_v
        self._steps_start_h =  steps_start_h
        self._steps_step_h =  steps_step_h
        self._steps_start_v =  steps_start_v
        self._steps_step_v =  steps_step_v
        self._number_of_points_h =  number_of_points_h
        self._number_of_points_v =  number_of_points_v
        self._energy =  energy
        self._sigma_h =  sigma_h
        self._sigma_v =  sigma_v
        self._amplitude =  amplitude
        self._kind_of_wave =  kind_of_wave
        self._n_h =  n_h
        self._n_v =  n_v
        self._beta_h =  beta_h
        self._beta_v =  beta_v

        self._units = units
        self._wavelength = wavelength
        self._initialize_amplitude = initialize_amplitude
        self._complex_amplitude_re = complex_amplitude_re
        self._complex_amplitude_im = complex_amplitude_im
        self._phase = phase
        self._radius = radius
        self._center = center
        self._inclination = inclination
        self._gaussian_shift = gaussian_shift
        self._add_random_phase = add_random_phase

    def get_dimension(self):
        return self._dimension

    # from Wofry Decorator
    def get_wavefront(self):
        if self._initialize_from == 0:
            if self._dimension == 1:
                wf = GenericWavefront1D.initialize_wavefront_from_range(
                    x_min=self._range_from_h, x_max=self._range_to_h,
                    number_of_points=self._number_of_points_h)
            elif self._dimension == 2:
                wf = GenericWavefront2D.initialize_wavefront_from_range(
                    x_min=self._range_from_h, x_max=self._range_to_h,
                    y_min=self._range_from_v, y_max=self._range_to_v,
                    number_of_points=(self._number_of_points_h, self._number_of_points_v))
        else:
            if self._dimension == 1:
                wf = GenericWavefront1D.initialize_wavefront_from_steps(
                    x_start=self._steps_start_h, x_step=self._steps_step_h,
                    number_of_points=self._number_of_points_h)
            elif self._dimension == 2:
                wf = GenericWavefront2D.initialize_wavefront_from_steps(
                    x_start=self._steps_start_h, x_step=self._steps_step_h,
                    y_start=self._steps_start_v, y_step=self._steps_step_v,
                    number_of_points=(self._number_of_points_h, self._number_of_points_v))


        if self._units == 0:
            wf.set_photon_energy(self._energy)
        else:
            wf.set_wavelength(self._wavelength)

        if self._kind_of_wave == 0:  # plane
            if self._dimension == 1:
                if self._initialize_amplitude == 0:
                    wf.set_plane_wave_from_complex_amplitude(complex_amplitude=complex(
                        self._complex_amplitude_re, self._complex_amplitude_im), inclination=self._inclination)
                else:
                    wf.set_plane_wave_from_amplitude_and_phase(amplitude=self._amplitude, phase=self._phase,
                                                                             inclination=self._inclination)
            elif self._dimension == 2:
                if self._initialize_amplitude == 0:
                    wf.set_plane_wave_from_complex_amplitude(complex_amplitude=complex(
                        self._complex_amplitude_re, self._complex_amplitude_im))
                else:
                    wf.set_plane_wave_from_amplitude_and_phase(amplitude=self._amplitude, phase=self._phase)

        elif self._kind_of_wave == 1:  # spheric
            wf.set_spherical_wave(radius=self._radius, center=self._center,
                                                complex_amplitude=complex(self._complex_amplitude_re,
                                                                          self._complex_amplitude_im))
        elif self._kind_of_wave == 2:  # gaussian
            if self._dimension == 1:
                wf.set_gaussian(sigma_x=self._sigma_h, amplitude=self._amplitude, shift=self._gaussian_shift)
            elif self._dimension == 2:
                wf.set_gaussian(sigma_x=self._sigma_h, sigma_y=self._sigma_v, amplitude=self._amplitude)
        elif self._kind_of_wave == 3:  # g.s.m.
            if self._dimension == 1:
                wf.set_gaussian_hermite_mode(
                                            sigma_x=self._sigma_h,
                                            mode_x=self._n_h,
                                            amplitude=self._amplitude,
                                            beta=self._beta_h,
                                            shift=self._gaussian_shift,
                                            )
            elif self._dimension == 2:
                wf.set_gaussian_hermite_mode(
                                            sigma_x=self._sigma_h,
                                            sigma_y=self._sigma_v,
                                            amplitude=self._amplitude,
                                            nx=self._n_h,
                                            ny=self._n_v,
                                            betax=self._beta_h,
                                            betay=self._beta_v,
                                            )

        if self._add_random_phase:
            wf.add_phase_shifts(2 * numpy.pi * numpy.random.random(wf.size()))


        return wf

    def to_python_code(self, do_plot=True):

        txt = ""

        txt += "#"
        txt += "\n# create output_wavefront\n#"
        txt += "\n#"

        if self._dimension == 1:
            txt += "\nfrom wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D"

            if self._initialize_from == 0:
                txt += "\noutput_wavefront = GenericWavefront1D.initialize_wavefront_from_range(x_min=%g,x_max=%g,number_of_points=%d)"%\
                (self._range_from_h,self._range_to_h,self._number_of_points_h)

            else:
                txt += "\noutput_wavefront = GenericWavefront1D.initialize_wavefront_from_steps(x_start=%g, x_step=%g,number_of_points=%d)"%\
                       (self._steps_start_h,self._steps_step_h,self._number_of_points_h)
        elif self._dimension == 2:
            txt += "\nfrom wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D"

            if self._initialize_from == 0:
                txt += "\noutput_wavefront = GenericWavefront2D.initialize_wavefront_from_range(x_min=%g,x_max=%g,y_min=%g,y_max=%g,number_of_points=(%d,%d))" % \
                       (self._range_from_h, self._range_to_h, self._range_from_v,  self._range_to_v, self._number_of_points_h, self._number_of_points_v)

            else:
                txt += "\noutput_wavefront = GenericWavefront2D.initialize_wavefront_from_steps(x_start=%g,x_step=%g,y_start=%g,y_step=%g,number_of_points=(%d,%d))" % \
                       (self._steps_start_h, self._steps_step_h, self._steps_start_v, self._steps_step_v, self._number_of_points_h, self._number_of_points_v)

        if self._units == 0:
            txt += "\noutput_wavefront.set_photon_energy(%g)"%(self._energy)
        else:
            txt += "\noutput_wavefront.set_wavelength(%g)"%(self._wavelength)



        if self._kind_of_wave == 0: #plane
            if self._dimension == 1:
                if self._initialize_amplitude == 0:
                    txt += "\noutput_wavefront.set_plane_wave_from_complex_amplitude(complex_amplitude=complex(%g,%g),inclination=%g)"%\
                           (self._complex_amplitude_re, self._complex_amplitude_im, self._inclination)
                else:
                    txt += "\noutput_wavefront.set_plane_wave_from_amplitude_and_phase(amplitude=%g,phase=%g,inclination=%g)"%(self.amplitude,self.phase,self.inclination)
            elif self._dimension == 2:
                if self._initialize_amplitude == 0:
                    txt += "\noutput_wavefront.set_plane_wave_from_complex_amplitude(complex_amplitude=complex(%g,%g))"%\
                           (self._complex_amplitude_re, self._complex_amplitude_im)
                else:
                    txt += "\noutput_wavefront.set_plane_wave_from_amplitude_and_phase(amplitude=%g,phase=%g)"%(self.amplitude,self.phase)

        elif self._kind_of_wave == 1: # spheric
            txt += "\noutput_wavefront.set_spherical_wave(radius=%g,center=%g,complex_amplitude=complex(%g, %g))"%\
                   (self._radius, self._center, self._complex_amplitude_re, self._complex_amplitude_im)
        elif self._kind_of_wave == 2: # gaussian
            if self._dimension == 1:
                txt += "\noutput_wavefront.set_gaussian(sigma_x=%f, amplitude=%f,shift=%f)"%\
                       (self._sigma_h, self._amplitude, self._gaussian_shift)
            elif self._dimension == 2:
                txt += "\noutput_wavefront.set_gaussian(sigma_x=%f, sigma_y=%f, amplitude=%f)"%\
                       (self._sigma_h, self._sigma_v, self._amplitude)

        elif self._kind_of_wave == 3: # g.s.m.
            if self._dimension == 1:
                txt += "\noutput_wavefront.set_gaussian_hermite_mode(sigma_x=%g,amplitude=%g,mode_x=%d,shift=%f,beta=%g)"%\
                       (self._sigma_h, self._amplitude, self._n_h, self._gaussian_shift, self._beta_h)
            elif self._dimension == 2:
                txt += "\noutput_wavefront.set_gaussian_hermite_mode(sigma_x=%g,sigma_y=%g,amplitude=%g,nx=%d,ny=%d,betax=%g,betay=%g)"%\
                       (self._sigma_h, self._sigma_v, self._amplitude, self._n_h, self._n_v, self._beta_h, self._beta_v)

        if self._dimension == 1:
            if self._add_random_phase:
                txt += "\noutput_wavefront.add_phase_shifts(2*numpy.pi*numpy.random.random(output_wavefront.size()))"

        if do_plot:
            if self._dimension == 1:
                txt += "\n\n\nfrom srxraylib.plot.gol import plot"
                txt += "\nplot(output_wavefront.get_abscissas(),output_wavefront.get_intensity(),title='SOURCE')"
            elif self._dimension == 2:
                txt += "\n\n\nfrom srxraylib.plot.gol import plot_image"
                txt += "\nplot_image(output_wavefront.get_intensity(),output_wavefront.get_coordinate_x(),output_wavefront.get_coordinate_y(),aspect='auto',title='SOURCE')"

        return txt

if __name__ == "__main__":

    ls = WOLightSource()


import numpy
from scipy.interpolate import interp2d
import scipy.constants as codata
import xraylib

from oasys.util.oasys_util import write_surface_file, read_surface_file
from oasys.util.oasys_objects import OasysSurfaceData


from syned.beamline.optical_element import OpticalElement
from syned.widget.widget_decorator import WidgetDecorator

from wofry.beamline.decorators import OpticalElementDecorator
from wofry.propagator.polarization import Polarization

from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D
from wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D


# mimics a syned element
# class Toolbox(OpticalElement):  # to be written the 2D version....


class WOToolbox1D(OpticalElement, OpticalElementDecorator):
    def __init__(self,
                 name="Undefined",
                 shift_center=0.0,
                 crop_factor=1.0,
                 abscissas_factor=1.0,  # abscissas_factor abscissas: 0=N0, 1=Yes
                 change_photon_energy=0,  # 0=No, 1=Yes
                 new_photon_energy=0.0,  # if change_photon_energy, the new photon energy in eV
                 ):

        super().__init__(
                      name=name)

        self._shift_center = shift_center
        self._crop_factor = crop_factor
        self._abscissas_factor = abscissas_factor
        self._change_photon_energy = change_photon_energy
        self._new_photon_energy = new_photon_energy

        # support text contaning name of variable, help text and unit. Will be stored in self._support_dictionary
        self._set_support_text([
                    ("name",                       "Name" ,                                "" ),
                    ("crop_factor",                "Crop factor",                          "" ),
                    ("abscissas_factor",           "Abscissas factor",                     ""),
                    ("change_photon_energy",       "Change photon energy",                 "(flag)" ),
                    ("new_photon_energy",          "New photon energy",                    "(if change_photon_energy>0)" ),
            ] )

    def get_shift_center(self):
        return self._shift_center

    def get_crop_factor(self):
        return self._crop_factor

    def get_abscissas_factor(self):
        return self._abscissas_factor

    def applyOpticalElement(self, input_wavefront, parameters=None, element_index=None):
        # return wavefront

        sigma = input_wavefront.get_complex_amplitude(polarization=Polarization.SIGMA)
        try:
            pi = input_wavefront.get_complex_amplitude(polarization=Polarization.PI)
        except:
            pi = None

        xnew = input_wavefront.get_abscissas().copy()

        xnew += self._shift_center

        if self._crop_factor > 1: # pad
            pad_width = int (input_wavefront.size() * self.get_crop_factor() ) -  input_wavefront.size()
            step = xnew[1] - xnew[0]


            if pad_width > 0:
                pad_halfwidth = pad_width // 2
                xnew = numpy.concatenate((
                    numpy.flip( (numpy.arange(pad_halfwidth) + 1) * step * (-1) ) + xnew[0],
                    xnew,
                    (numpy.arange(pad_halfwidth) + 1) * step + xnew[-1],
                ))
                sigma = numpy.pad(sigma, (pad_halfwidth, pad_halfwidth), 'constant', constant_values=(0, 0))
                if pi is not None:
                    pi = numpy.pad(pi, (pad_halfwidth, pad_halfwidth), 'constant', constant_values=(0, 0))

        elif self._crop_factor < 1: #crop
            crop_halfwidth = int(input_wavefront.size() * (1.0 - self.get_crop_factor())  / 2)

            if crop_halfwidth > 0:
                xnew = xnew[crop_halfwidth:-crop_halfwidth]
                sigma = sigma[crop_halfwidth:-crop_halfwidth]
                if pi is not None:
                    pi = pi[crop_halfwidth:-crop_halfwidth]
        else:
            pass

        if self.get_abscissas_factor() != 1.0:
            xnew *= self.get_abscissas_factor()

        output_wavefront = GenericWavefront1D.initialize_wavefront_from_arrays(
            xnew, sigma, y_array_pi=pi, wavelength=input_wavefront.get_wavelength())

        if self._change_photon_energy:
            output_wavefront.set_photon_energy(self._new_photon_energy)

        return output_wavefront

    def to_python_code(self, data=None):
        txt  = ""
        txt += "\nfrom orangecontrib.esrf.wofry.util.toolbox import WOToolbox1D #TODO update"
        txt += "\n"

        txt += "\noptical_element = WOToolbox1D(name='%s',crop_factor=%g,abscissas_factor=%d,shift_center=%g,change_photon_energy=%d,new_photon_energy=%g)" % \
               (self.get_name(), self.get_crop_factor(), self.get_abscissas_factor(), self.get_shift_center(), self._change_photon_energy, self._new_photon_energy)

        txt += "\n"
        return txt

if __name__ == "__main__":

    import numpy
    from srxraylib.plot.gol import plot, plot_image


    #
    # 2D
    #
    if False:
        input_wavefront = GenericWavefront2D.initialize_wavefront_from_range(x_min=-0.0003, x_max=0.0003, y_min=-0.0003,
                                                                              y_max=0.0003, number_of_points=(400, 200))
        input_wavefront.set_photon_energy(10000)
        input_wavefront.set_plane_wave_from_complex_amplitude(complex_amplitude=complex(1, 0))

        optical_element = WOThinObject(name='ThinObject',
                                       file_with_thickness_mesh='/home/srio/Downloads/SRW_M_thk_res_workflow_a_FC_CDn01.dat.h5',
                                       material='Be')

        # no drift in this element
        output_wavefront = optical_element.applyOpticalElement(input_wavefront)

        #
        # ---- plots -----
        #
        plot_image(output_wavefront.get_intensity(), output_wavefront.get_coordinate_x(),
                   output_wavefront.get_coordinate_y(), aspect='auto', title='OPTICAL ELEMENT NR 1')


    #
    # 1D
    #

    if True:

        input_wavefront = GenericWavefront1D.initialize_wavefront_from_range(x_min=-0.004, x_max=0.004, number_of_points=1000)
        input_wavefront.set_wavelength(1e-10)
        input_wavefront.set_gaussian(sigma_x=0.001, amplitude=1, shift=0)

        optical_element = WOToolbox1D(name='test', shift_center=0.002, crop_factor=0.6, abscissas_factor=1,
                                      change_photon_energy=0, new_photon_energy=0.0)


        # no drift in this element
        output_wavefront = optical_element.applyOpticalElement(input_wavefront)
        #
        # ---- plots -----
        #
        plot(input_wavefront.get_abscissas(), input_wavefront.get_intensity(),
            output_wavefront.get_abscissas(), output_wavefront.get_intensity(),
                   title='OPTICAL ELEMENT NR 1', legend=['input',
                                                         'output'])


import numpy
from PyQt5.QtWidgets import QApplication, QMessageBox, QSizePolicy

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence
from oasys.widgets.exchange import DataExchangeObject

from xoppylib.power.xoppy_calc_power import xoppy_calc_power

from oasys.widgets.exchange import DataExchangeObject
from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget

import scipy.constants as codata

import xraylib
from dabax.dabax_xraylib import DabaxXraylib

class OWID24HPAttenuators(XoppyWidget):
    name = "ID24_HP_Attenuators"
    id = "orange.widgets.dataxpower"
    description = "Power Absorbed and Transmitted by Optical Elements"
    icon = "icons/id24_hp_attenuator.png"
    priority = 2
    category = ""
    keywords = ["xoppy", "power"]

    inputs = [("ExchangeData", DataExchangeObject, "acceptExchangeData")]

    SOURCE = Setting(2)
    ENER_MIN = Setting(1000.0)
    ENER_MAX = Setting(50000.0)
    ENER_N = Setting(100)
    SOURCE_FILE = Setting("?")
    Axis1 = Setting(0)
    Axis2 = Setting(0)
    Axis3 = Setting(0)    
    

    PLOT_SETS = Setting(0)
    FILE_DUMP = 0

    MATERIAL_CONSTANT_LIBRARY_FLAG = Setting(0) # not yet interfaced, to be done

    input_spectrum = None
    input_script = None

    def __init__(self):
        super().__init__(show_script_tab=True)

    def build_gui(self):

        self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
        self.leftWidgetPart.setMaximumWidth(self.CONTROL_AREA_WIDTH + 20)
        self.leftWidgetPart.updateGeometry()

        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)

        idx = -1 

        #widget index 2 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.box_source = gui.comboBox(box1, self, "SOURCE",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['From Oasys wire', 'Normalized to 1 W/eV', 'From external file (eV, W/eV)', 'From external file (eV, phot/s/.1%bw)'],
                    valueType=int, orientation="horizontal", labelWidth=150)
        self.show_at(self.unitFlags()[idx], box1)
        
        #widget index 6 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_MIN",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 7 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_MAX",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 8 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_N",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 9 ***********   File Browser ******************
        idx += 1
        box1 = gui.widgetBox(box)
        file_box_id = oasysgui.widgetBox(box1, "", addSpace=False, orientation="horizontal")
        self.file_id = oasysgui.lineEdit(file_box_id, self, "SOURCE_FILE", self.unitLabels()[idx],
                                    labelWidth=100, valueType=str, orientation="horizontal")
        gui.button(file_box_id, self, "...", callback=self.select_input_file, width=25)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 10
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "Axis1",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['None', 'Diam: 0.3 mm','PyroC: 3 mm', 'Diam: 0.3 mm'],
                    valueType=str, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 11
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "Axis2",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['None', 'Al: 0.5 mm + BlackDiam: 0.5 mm','PyroC: 1 mm + BlackDiam: 0.5 mm', 'PyroC: 2 mm + PyroC: 0.5 mm'],
                    valueType=str, orientation="horizontal", labelWidth=175, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 12
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "Axis3",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['None', 'Al: 1 mm + BlackDiam: 0.5 mm', 'PyroC: 0.5 mm + BlackDiam: 0.5 mm', 'PyroC: 3 mm'],
                    valueType=str, orientation="horizontal", labelWidth=150, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)        

        #widget index 13
        idx += 1
        box1 = gui.widgetBox(box)
        gui.separator(box1, height=7)

        gui.comboBox(box1, self, "PLOT_SETS",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['Total Attenuation', 'Axis Attenuation'],
                    valueType=int, orientation="horizontal", labelWidth=250, callback=self.set_EL_FLAG)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 14
        idx += 1
        box1 = gui.widgetBox(box)
        gui.separator(box1, height=7)

        gui.comboBox(box1, self, "FILE_DUMP",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['No', 'Yes (power.spec)'],
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        #self.input_spectrum = None

    def select_input_file(self):
        self.file_id.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE,
                                    "Open 2-columns file with spectral power",
                                    file_extension_filter="ascii dat (*.dat *.txt *spec)"))   

    def set_EL_FLAG(self):
        self.initializeTabs()

    def unitLabels(self):
         return ['Input beam:',
                 'From energy [eV]:      ',
                 'To energy [eV]:',
                 'Energy points:  ',
                 'File with input beam spectral power:',
                 'Axis1','Axis2','Axis3',                                  
                 'Plot','Dump file']


    def unitFlags(self):
         return ['True',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  >  1',
                 'True','True','True',                 
                 'True','True']

    def get_help_name(self):
        return 'power'

    def selectFile(self):
        self.le_source_file.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE, "Open Source File", file_extension_filter="*.*"))

    def acceptExchangeData(self, exchangeData):

        self.input_spectrum = None
        self.input_script = None
        self.SOURCE = 0

        try:
            if not exchangeData is None:
                if exchangeData.get_program_name() == "XOPPY":
                    no_bandwidth = False
                    if exchangeData.get_widget_name() =="UNDULATOR_FLUX" :
                        # self.SOURCE_FILE = "xoppy_undulator_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() == "BM" :
                        if exchangeData.get_content("is_log_plot") == 1:
                            raise Exception("Logaritmic X scale of Xoppy Energy distribution not supported")
                        if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") in [0,2]:
                            # self.SOURCE_FILE = "xoppy_bm_flux"
                            no_bandwidth = True
                            index_flux = 6
                        else:
                            raise Exception("Xoppy result is not a Flux vs Energy distribution integrated in Psi")
                    elif exchangeData.get_widget_name() =="XWIGGLER" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="WS" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="XTUBES" :
                        # self.SOURCE_FILE = "xoppy_xtubes_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="XTUBE_W" :
                        # self.SOURCE_FILE = "xoppy_xtube_w_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="BLACK_BODY" :
                        # self.SOURCE_FILE = "xoppy_black_body_flux"
                        no_bandwidth = True
                        index_flux = 2

                    elif exchangeData.get_widget_name() =="UNDULATOR_RADIATION" :
                        # self.SOURCE_FILE = "xoppy_undulator_radiation"
                        no_bandwidth = True
                        index_flux = 1
                    elif exchangeData.get_widget_name() =="POWER" :
                        # self.SOURCE_FILE = "xoppy_undulator_power"
                        no_bandwidth = True
                        index_flux = -1
                    elif exchangeData.get_widget_name() =="POWER3D" :
                        # self.SOURCE_FILE = "xoppy_power3d"
                        no_bandwidth = True
                        index_flux = 1

                    else:
                        raise Exception("Xoppy Source not recognized")

                    # self.SOURCE_FILE += "_" + str(id(self)) + ".dat"


                    spectrum = exchangeData.get_content("xoppy_data")

                    if exchangeData.get_widget_name() =="UNDULATOR_RADIATION" or \
                        exchangeData.get_widget_name() =="POWER3D":
                        [p, e, h, v ] = spectrum
                        tmp = p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*codata.e*1e3
                        spectrum = numpy.vstack((e,p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*
                                                 codata.e*1e3))
                        self.input_spectrum = spectrum
                    else:

                        if not no_bandwidth:
                            spectrum[:,index_flux] /= 0.001*spectrum[:,0]

                        self.input_spectrum = numpy.vstack((spectrum[:,0],spectrum[:,index_flux]))

                    try:
                        self.input_script = exchangeData.get_content("xoppy_script")
                    except:
                        self.input_script = None

                    self.process_showers()
                    self.compute()

        except Exception as exception:
            QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)

    def check_fields(self):

        if self.SOURCE == 1:
            self.ENER_MIN = congruence.checkPositiveNumber(self.ENER_MIN, "Energy from")
            self.ENER_MAX = congruence.checkStrictlyPositiveNumber(self.ENER_MAX, "Energy to")
            congruence.checkLessThan(self.ENER_MIN, self.ENER_MAX, "Energy from", "Energy to")
            self.NPOINTS = congruence.checkStrictlyPositiveNumber(self.ENER_N, "Energy Points")
        elif self.SOURCE == 2:
            congruence.checkFile(self.SOURCE_FILE)

    ### Input for each axis of HP attenuators    

    def att_substance(self):
        """ Substance (or material) for each axis """
        substance=[]
        # Axis 1
        if self.Axis1 == 0: substance.append('C') #None
        if self.Axis1 == 1: substance.append('C') #Diamond
        if self.Axis1 == 2: substance.append('C') #PyroC
        if self.Axis1 == 3: substance.append('C') #Diamond
        # Axis 2
        if self.Axis2 == 0: substance.append('C')          #None
        if self.Axis2 == 1: substance.extend(('Al', 'C'))  #Alu + Black Diamond 
        if self.Axis2 == 2: substance.extend(('C', 'C'))   #PyroC + Black Diamond
        if self.Axis2 == 3: substance.extend(('C', 'C'))   #PyroC + PyroC
        # Axis 3
        if self.Axis3 == 0: substance.append('C')          #None
        if self.Axis3 == 1: substance.extend(('Al', 'C'))  #Alu + Black Diamond  
        if self.Axis3 == 2: substance.extend(('C', 'C'))   #PyroC + Black Diamond
        if self.Axis3 == 3: substance.append('C')          #PyroC

        return substance
    
    def att_thick(self):
        """ Thickness of each attenuator at a given axis """
        thick=[]
        # Axis 1
        if self.Axis1 == 0: thick.append(0.0) #None
        if self.Axis1 == 1: thick.append(0.3) #Diamond
        if self.Axis1 == 2: thick.append(3.0) #PyroC
        if self.Axis1 == 3: thick.append(0.3) #Diamond
        # Axis 2
        if self.Axis2 == 0: thick.append(0.0)        #None
        if self.Axis2 == 1: thick.extend((0.5, 0.5)) #Al + Black Diamond 
        if self.Axis2 == 2: thick.extend((1.0, 0.5)) #PyroC + Black Diamond
        if self.Axis2 == 3: thick.extend((2.0, 0.5)) #PyroC + PyroC
        # Axis 3
        if self.Axis3 == 0: thick.append(0.0)        #None        
        if self.Axis3 == 1: thick.extend((1.0, 0.5)) #Al + Black Diamond        
        if self.Axis3 == 2: thick.extend((0.5, 0.5)) #PyroC + Black Diamond
        if self.Axis3 == 3: thick.append(3.0)        #PyroC

        return thick    

    def att_dens(self):
        """ Particular density of each attenuator at a given axis """
        dens = []
        # Axis 1
        if self.Axis1 == 0: dens.append(3.52) #None
        if self.Axis1 == 1: dens.append(3.52) #Diamond
        if self.Axis1 == 2: dens.append(1.40) #PyroC
        if self.Axis1 == 3: dens.append(3.52) #Diamond
        # Axis 2
        if self.Axis2 == 0: dens.append(3.52)       #None
        if self.Axis2 == 1: dens.extend((2.7, 3.5)) #Alu + Black Diamond 
        if self.Axis2 == 2: dens.extend((1.4, 3.5)) #PyroC + Black Diamond
        if self.Axis2 == 3: dens.extend((1.4, 1.4)) #PyroC + PyroC
        # Axis 3
        if self.Axis3 == 0: dens.append(3.52)       #None        
        if self.Axis3 == 1: dens.extend((2.7, 3.5)) #Alu + Black Diamond  
        if self.Axis3 == 2: dens.extend((1.4, 3.5)) #PyroC + Black Diamond
        if self.Axis3 == 3: dens.append(1.4)        #PyroC       

        return dens        

    def do_xoppy_calculation(self):

        if self.SOURCE == 0:
            if self.input_spectrum is None:
                raise Exception("No input beam")
            else:
                energies = self.input_spectrum[0,:].copy()
                source = self.input_spectrum[1,:].copy()
            if self.input_script is None:
                script_previous = '#\n# >> MISSING SCRIPT TO CREATE (energy, spectral_power) <<\n#\n'
            else:
                script_previous = self.input_script
        elif self.SOURCE == 1:
            energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
            source = numpy.ones(energies.size)
            tmp = numpy.vstack( (energies,source))
            self.input_spectrum = source
            script_previous = "import numpy\nenergy = numpy.linspace(%g,%g,%d)\nspectral_power = numpy.ones(%d)\n" % \
                        (self.ENER_MIN,self.ENER_MAX,self.ENER_N,self.ENER_N)
        elif self.SOURCE == 2:  # file contains energy_eV and spectral power (W/eV)
            source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1]
                self.input_spectrum = source
                script_previous = "import numpy\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1]\n" % \
                                (source_file)
            except:
                print("Error loading file %s "%(source_file))
                raise
        elif self.SOURCE == 3:  # file contains energy_eV and flux (ph/s/0.1%bw
            source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1] * (codata.e * 1e3)
                self.input_spectrum = source
                script_previous = "import numpy\nimport scipy.constants as codata\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1] / (codata.e * 1e3)\n" % \
                                (source_file)
            except:
                print("Error loading file %s "%(source_file))
                raise        

        substance = self.att_substance()  # str
        thick     = self.att_thick() # float
        angle     = numpy.zeros_like(thick) # float
        dens      = self.att_dens() # str
        roughness = numpy.zeros_like(dens) # float
        flags     = numpy.zeros_like(dens) # int

        NELEMENTS = len(self.att_substance())

        # this is for creating script
        substance_str = "["
        thick_str = "["
        angle_str = "["
        dens_str = "["
        roughness_str = "["
        flags_str = "["
        for i in range(NELEMENTS):
            substance_str += "'%s'," % (substance[i])
            thick_str     += "%g," % (thick[i])
            angle_str     += "%g," % (angle[i])
            dens_str      += "'%s'," % (dens[i])
            roughness_str += "%g," % (roughness[i])
            flags_str     += "%d," % (flags[i])
        substance_str += "]"
        thick_str += "]"
        angle_str += "]"
        dens_str += "]"
        roughness_str += "]"
        flags_str += "]"

        if self.MATERIAL_CONSTANT_LIBRARY_FLAG == 0:
            material_constants_library = xraylib
            material_constants_library_str = "xraylib"
        else:
            material_constants_library = DabaxXraylib()
            material_constants_library_str = 'DabaxXraylib()'
            print(material_constants_library.info())

        out_dictionary = xoppy_calc_power(
            energies,
            source,
            substance                  = substance,
            thick                      = thick    ,
            angle                      = angle    ,
            dens                       = dens     ,
            roughness                  = roughness,
            flags                      = flags    ,
            nelements                  = NELEMENTS,
            FILE_DUMP                  = self.FILE_DUMP,
            material_constants_library = material_constants_library,
                                                )

        print(out_dictionary["info"])

        dict_parameters = {
            "substance"                 : substance_str,
            "thick"                     : thick_str,
            "angle"                     : angle_str,
            "dens"                      : dens_str,
            "roughness"                 : roughness_str,
            "flags"                     : flags_str,
            "nelements"                 : NELEMENTS,
            "FILE_DUMP"                 : self.FILE_DUMP,
            "material_constants_library": material_constants_library_str,
        }

        script_element = self.script_template().format_map(dict_parameters)

        script = script_previous + script_element

        self.xoppy_script.set_code(script)

        return  out_dictionary, script

    def script_template(self):
        return """
#
# script to make the calculations (created by XOPPY:xpower)
#
import numpy
from xoppylib.power.xoppy_calc_power import xoppy_calc_power
import xraylib
from dabax.dabax_xraylib import DabaxXraylib
out_dictionary = xoppy_calc_power(
        energy,
        spectral_power,
        substance = {substance},
        thick     = {thick}, # in mm (for filters)
        angle     = {angle}, # in mrad (for mirrors)
        dens      = {dens},
        roughness = {roughness}, # in A (for mirrors)
        flags     = {flags}, # 0=Filter, 1=Mirror
        nelements = {nelements},
        FILE_DUMP = {FILE_DUMP},
        material_constants_library = {material_constants_library},
        )
# data to pass
energy = out_dictionary["data"][0,:]
spectral_power = out_dictionary["data"][-1,:]
#                       
# example plots
#
from srxraylib.plot.gol import plot
plot(out_dictionary["data"][0,:], out_dictionary["data"][1,:],
    out_dictionary["data"][0,:], out_dictionary["data"][-1,:],
    xtitle=out_dictionary["labels"][0],
    legend=[out_dictionary["labels"][1],out_dictionary["labels"][-1]],
    title='Spectral Power [W/eV]')
 
#
# end script
#
"""  
    def extract_data_from_xoppy_output(self, calculation_output):

        """ Custom calculated data for this widget depending on which total HP power
        or power by HP attenuator axis is needed"""        

        out_dictionary, script = calculation_output

        cumulated_data = {}
        results=[]
        shift = 6 
        
        # First, we add the typical: energy axis and input beam
        
        results.append((out_dictionary['data'][0]).tolist()) #Pos: 0
        results.append((out_dictionary['data'][1]).tolist()) #Pos: 1

        #if just the total of the HP attenuators is needed

        if self.do_plot_total():
            #Total
            total_trans = numpy.ones_like(out_dictionary['data'][0])
            total_abs_spectra = numpy.zeros_like(out_dictionary['data'][0])
            for i in range(len(self.att_substance())):
                total_trans *= out_dictionary['data'][4 + i * shift]
                total_abs_spectra += out_dictionary['data'][6 + i * shift]

            results.append(total_trans.tolist())                           #Pos: 2
            results.append(total_abs_spectra.tolist())                     #Pos: 3
            results.append((out_dictionary['data'][-1]).tolist())          #Pos: 4

        # if what is needed is the absorption per axis
        
        if self.do_plot_per_axis():              
            #Axis 1                
            results.append((out_dictionary['data'][4]).tolist()) #Trasmitivity            #Pos: 2
            results.append((out_dictionary['data'][6]).tolist()) #Spectral power absorbed #Pos: 3
            results.append((out_dictionary['data'][7]).tolist()) #Spectral power after    #Pos: 4
    
            #Axis 2
            
            if self.Axis2 == 0:
                results.append((out_dictionary['data'][4 + shift]).tolist()) #Pos: 5
                results.append((out_dictionary['data'][6 + shift]).tolist()) #Pos: 6
                results.append((out_dictionary['data'][7 + shift]).tolist()) #Pos: 7
                                                
            elif self.Axis2 >= 1:
                results.append((out_dictionary['data'][4 + shift] * out_dictionary['data'][4 + 2 * shift]).tolist()) #Pos: 5
                results.append((out_dictionary['data'][6 + shift] + out_dictionary['data'][6 + 2 * shift]).tolist()) #pos: 6
                results.append((out_dictionary['data'][7 + 2 * shift]).tolist())                                     #Pos: 7       
    
            #Axis 3
    
            if self.Axis3 == 0 or self.Axis3 == 3:
                
                if self.Axis2 == 0: # None in Axis2
                    results.append((out_dictionary['data'][4 + 2 * shift]).tolist()) #Pos: 8
                    results.append((out_dictionary['data'][6 + 2 * shift]).tolist()) #Pos: 9
                    results.append((out_dictionary['data'][7 + 2 * shift]).tolist()) #Pos: 10         
            
                elif self.Axis2 >= 1:
                    results.append((out_dictionary['data'][4 + 3 * shift]).tolist()) #Pos: 8
                    results.append((out_dictionary['data'][6 + 3 * shift]).tolist()) #Pos: 9
                    results.append((out_dictionary['data'][7 + 3 * shift]).tolist()) #Pos: 10           
    
            elif 0 < self.Axis3 < 3 :
                
                if self.Axis2 == 0: # None in Axis2
                    results.append((out_dictionary['data'][4 + 2 * shift] * out_dictionary['data'][4 + 3 * shift]).tolist()) #Pos: 8
                    results.append((out_dictionary['data'][6 + 2 * shift] + out_dictionary['data'][6 + 3 * shift]).tolist()) #Pos: 9
                    results.append((out_dictionary['data'][7 + 3 * shift]).tolist())                                         #Pos: 10 
                elif self.Axis2 >= 1:
                    results.append((out_dictionary['data'][4 + 3 * shift] * out_dictionary['data'][4 + 4 * shift]).tolist()) #Pos: 8
                    results.append((out_dictionary['data'][6 + 3 * shift] + out_dictionary['data'][6 + 4 * shift]).tolist()) #Pos: 9
                    results.append((out_dictionary['data'][7 + 4 * shift]).tolist())                                         #Pos: 10

        cumulated_data['data']=numpy.array(results)

        # send exchange
        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
        
        try:
            calculated_data.add_content("xoppy_data", cumulated_data["data"].T)
        except:
            pass
        try:
            calculated_data.add_content("xoppy_script", script)
        except:
            pass
        try:
            calculated_data.add_content("labels", out_dictionary["labels"])
        except:
            pass
        try:
           calculated_data.add_content("info", out_dictionary["info"])
        except:
           pass

        return calculated_data


    def get_data_exchange_widget_name(self):
        return "POWER"

    def do_plot_total(self):
        """ When HP total properties """
        out = False
        if self.PLOT_SETS == 0: out = True                        
        return out

    def do_plot_per_axis(self):
        """ When we want to check for each axis"""
        out = False
        if self.PLOT_SETS == 1: out = True        
        return out

    def getTitles(self):
        titles = []

        titles.append("Input beam") #input beam always

        if self.do_plot_total():
            
            titles.append("Total transmitivity of HP attenuators")
            titles.append("Total spectral power absorbed in HP attenuators")
            titles.append("Spectral power after HP attenuators")

        if self.do_plot_per_axis():
            for axis_n in range(3):
                titles.append("Transmitivity of Axis " + str(axis_n + 1))
                titles.append("Spectral power absorbed in Axis " + str(axis_n + 1))
                titles.append("Spectral power after Axis " + str(axis_n + 1))

        return titles

    def getXTitles(self):
        xtitles = []

        xtitles.append("Photon Energy [eV]") #for ithe input beam

        if self.do_plot_total():
            for axis_n in range(3):
                xtitles.append("Photon Energy [eV]")
        if self.do_plot_per_axis():
            for axis_n in range(3*3):
                xtitles.append("Photon Energy [eV]")    

        return xtitles

    def getYTitles(self):
        ytitles = []

        if self.SOURCE == 1:
            unit_str = '[a.u]'
        else:
            unit_str = '[W/eV]'

        ytitles.append("Spectral Power %s" % unit_str ) #input beam

        if self.do_plot_total():
            ytitles.append("Transmitivity")
            ytitles.append("Spectral power  %s" % unit_str)
            ytitles.append("Spectral power  %s" % unit_str)

        if self.do_plot_per_axis():            
            for axis_n in range(3):        
                ytitles.append("Transmitivity")
                ytitles.append("Spectral power  %s" % unit_str)
                ytitles.append("Spectral power  %s" % unit_str)        

        return ytitles

    def getVariablesToPlot(self):
        variables = []

        variables.append((0, 1))  # start plotting the source

        if self.do_plot_total():
            variables.append((0, 2))
            variables.append((0, 3))
            variables.append((0, 4))
            
        if self.do_plot_per_axis():
            for axis_n in range(3):
                variables.append((0, (2 + 3 * axis_n)))
                variables.append((0, (3 + 3 * axis_n)))
                variables.append((0, (4 + 3 * axis_n)))        

        return variables

    def getLogPlot(self):
        logplot = []

        logplot.append((False,False)) #source

        if self.do_plot_total():
            for axis_n in range(3):
                logplot.append((False,False))        
        
        if self.do_plot_per_axis:
            for axis_n in range(3*3):
                logplot.append((False, False))

        return logplot

if __name__ == "__main__":
    import sys
    input_type = 0

    if input_type == 1:
        from oasys.widgets.exchange import DataExchangeObject
        input_data_type = "POWER"

        if input_data_type == "POWER":
            # create fake UNDULATOR_FLUX xoppy exchange data
            e = numpy.linspace(1000.0, 10000.0, 100)
            source = e/10
            received_data = DataExchangeObject("XOPPY", "POWER")
            received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
            received_data.add_content("xoppy_code", "US")

        elif input_data_type == "POWER3D":
            # create unulator_radiation xoppy exchange data
            from xoppylib.sources.xoppy_undulators import xoppy_calc_undulator_radiation

            e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
                                               ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
                                               ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
                                               PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
                                               SETRESONANCE=0,HARMONICNUMBER=1,
                                               GAPH=0.001,GAPV=0.001,\
                                               HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
                                               PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
                                               USEEMITTANCES=1)
            received_data = DataExchangeObject("XOPPY", "POWER3D")
            received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
            received_data.add_content("xoppy_data", [p, e, h, v])
            received_data.add_content("xoppy_code", code)

        app = QApplication(sys.argv)
        w = OWID24HPAttenuators()
        w.acceptExchangeData(received_data)
        w.show()
        app.exec()
        w.saveSettings()

    else:
        app = QApplication(sys.argv)
        w = OWID24HPAttenuators()
        w.SOURCE = 1
        w.show()
        app.exec()
        w.saveSettings()
